import genj.gedcom.*;
import genj.report.*;

import java.io.*;

/**
 * GenJ - Report
 * @author Nils Meier nils@meiers.org
 * @version 0.1
 */
public class ReportMakeHTMLTable implements Report {

  /**
   * Returns the name of this report - should be localized.
   */
  public String getName() {
    return "Make HTML-Table";
  }

  /**
   * Some information about this report
   * @return Information as String
   */
  public String getInfo() {
    return "This report exports individuals' information to HTML";
  }

  /**
   * This method actually starts this report
   */
  public boolean start(ReportView view, Gedcom gedcom, ReportWriter out) throws InterruptedException {

    // HEAD
    out.println("<HTML>");
    out.println("<HEAD>");
    out.println("<TITLE>"+gedcom.getName()+" - HTML Table</TITLE>");
    out.println("</HEAD>");
    out.println("<BODY bgcolor=\"#ffffff\">");

    // TABLE
    out.println("<TABLE border=1 cellspacing=1>");

    // TABLE HEADER
    htmlCell(out,"Last Name");
    htmlCell(out,"First Name");
    htmlCell(out,"Sex");
    htmlCell(out,"Birth");
    htmlCell(out,"Place");
    htmlCell(out,"Death");
    htmlCell(out,"Place");
    
    // Go through individuals
    EntityList indis = gedcom.getEntities(Gedcom.INDIVIDUALS);

    for (int i=0;i<indis.getSize();i++) {
      
      out.println("<TR>");
      
      export(indis.getIndi(i),out);

      out.println("</TR>");

      // .. next individual
    }

    // END TABLE
    out.println("</TABLE>");
    
    // TAIL
    out.println("</BODY>");
    out.println("</HTML>");
    
    // Done
    return true;
  }

  /**
   * Export individual's information
   */
  private void export(Indi indi, ReportWriter out) throws InterruptedException {
    
    // Standard
    htmlCell(out,indi.getLastName() );
    htmlCell(out,indi.getFirstName());

    switch (indi.getSex()) {
    case Gedcom.MALE:
      htmlCell(out,"Male");
      break;
    case Gedcom.FEMALE:
      htmlCell(out,"Female");
      break;
    default:
      htmlCell(out,"&nbsp;");
      break;
    }

    // Birth Date
    try {
      // ... this might fail in case we don't get a PropertyEvent back
      PropertyEvent event = (PropertyEvent)indi.getProperty(new TagPath("BIRT"),true);
      PropertyDate date = (PropertyDate)event.getProperty(new TagPath("DATE"),true);
      PropertyPlace place = (PropertyPlace)event.getProperty(new TagPath("PLAC"),true);
      htmlCell(out,date);
      htmlCell(out,place);
    } catch (Exception e) {
      htmlCell(out,null);
      htmlCell(out,null);
    }

    try {
      PropertyEvent event = (PropertyEvent)indi.getProperty(new TagPath("DEAT"),true);
      PropertyDate date = (PropertyDate)event.getProperty(new TagPath("DATE"),true);
      PropertyPlace place = (PropertyPlace)event.getProperty(new TagPath("PLAC"),true);
      htmlCell(out,date);
      htmlCell(out,place);
    } catch (Exception e) {
      htmlCell(out,null);
      htmlCell(out,null);
    }


    // Done    
  }

  /** 
   * Writes HTML table cell information
   */ 
  private void htmlCell(ReportWriter out, Object content) throws InterruptedException {
    // We don't want to see 'null' but ''
    if (content == null)
       content = "&nbsp;";
    // We ask a property for it's value instead of just toString()
    if (content instanceof Property)
      content = ((Property)content).getValue();       
    // Here comes the HTML
    out.println("<TD>"+content+"</TD>");
  } 
  
  /**
   * Indication of how this reports shows information
   * to the user. Standard Out here only.
   */
  public boolean usesStandardOut() {
    return true;
  }

  /**
   * Author
   */
  public String getAuthor() {
    return "Nils Meier <nils@meiers.org>";
  }
  
  /**
   * Tells wether this report doesn't change information in the Gedcom-file
   */
  public boolean isReadOnly() {
    return true;
  }
}

