package genj.app;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.lang.reflect.Method;

import javax.swing.*;
import javax.swing.event.*;




import genj.gedcom.*;import genj.util.ImgIcon;/**
 * Component monitoring selected entities and showing the actual one
 */
public class EntitySelector extends JLabel implements GedcomListener {

  private Gedcom gedcom;
  private int filter[];
  private Entity entity;

  /**
   * Constructor
   */
  public EntitySelector() {

	// Initial look
	setBorder(BorderFactory.createLoweredBevelBorder());
	setEntity(null);

	// Done	
  }      
  /**
   * A helper checks wether an an entity matches the filter
   * @return wether it fits or not
   */
  private boolean checkFilter(Entity entity) {
	// No entity or filter - o.k.
	if ((entity==null)||(filter==null))
	  return true;
	// Loop through filter
	for (int i=0;i<filter.length;i++) {
	  if (filter[i]==entity.getType())
		return true;
	}
	// No candidate
	return false;
  }      
  /**
   * Returns the selected entity
   */
  public Entity getEntity() {
	return entity;
  }      
  /**
   * Notification of changes in Gedcom - we check
   * wether our entity has been removed so have to forget it
   */
  public void handleChange(Change change) {
	if ((entity!=null)&&(change.isChanged(change.EDEL))&&(change.getEntities(change.EDEL).contains(entity)))
	  setEntity(null);
  }      
  /**
   * Notification of closed Gedcom - we stop listening
   * to it
   */
  public void handleClose(Gedcom which) {
	setGedcom(null);
  }      
  /**
   * Notification of a selection in Gedcom - we remember
   * selected entity according to filter
   */
  public void handleSelection(Selection selection) {
	if (checkFilter(selection.getEntity()))
	  setEntity(selection.getEntity());
  }      
  /**
   * This control isnt used anymore
   */
  public void removeNotify() {
	super.removeNotify();
	setGedcom(null);
  }      
  /**
   * Selects an entity
   */
  public void setEntity(Entity pEntity) {
	// Remember
	entity=pEntity;
	// Show
	if (entity!=null) {
	  setIcon(entity.getProperty().getImage(false).getImageIcon());
	  setText(entity.toString());
	} else {
	  setIcon(PropertyIndi.getDefaultImage().getImageIcon());
	  setText("Unknown");
	}
	// Done
  }      
  /**
   * Sets the types this selector will accept as being selected
   */
  public void setFilter(int[] pFilter) {
	filter = pFilter;

	if (!checkFilter(entity))
	  setEntity(null);
  }      
  /**
   * Sets the Gedcom to listen to
   */
  public void setGedcom(Gedcom pGedcom) {
	// Stop listening
	if (gedcom!=null)
	  gedcom.removeListener(this);
	// Remember
	gedcom = pGedcom;
	// Start listening
	if (gedcom!=null)
	  gedcom.addListener(this);
	// Done
  }      
}