package genj.app;

 






import java.util.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import genj.*;import genj.gedcom.*;import genj.util.*;import awtx.table.*;import awtx.Table;/**
 * Component class for showing a table of all gedcoms
 */
public class GedcomTable extends Table {

  private GTableModel model;

  private static final Object defaultHeaders[] = {
	"Name"                             ,
	PropertyIndi     .getDefaultImage(),
	PropertyFam      .getDefaultImage(),
	PropertyMedia    .getDefaultImage(),
	PropertyNote     .getDefaultImage(),
	PropertySource   .getDefaultImage(),
	PropertySubmitter.getDefaultImage()
  };
  private static final int defaultWidths[] = {
	48,
	24,
	24,
	24,
	24,
	24,
	24
  };

  /**
   * Our header renderer
   */
  class GHeaderRenderer extends DefaultHeaderCellRenderer {

	public void render(Graphics g, Rectangle rect, Object o, FontMetrics fm) {

	  // Text?
	  if (o instanceof String) {
		// .. draw normal
		super.render(g,rect,o,fm);
		// .. done
		return;
	  }

	  // Image?
	  if (o instanceof ImgIcon) {
		// .. draw normal without object (text)
		super.render(g,rect,null,fm);
		// .. draw image
		g.drawImage(
		  ((ImgIcon)o).getImage(),
		  rect.x+4,
		  rect.y,
		  null
		);
		// .. done
		return;
	  }

	  // Done
	}

  }


  /**
   * Class for encapsulating all Gedcoms in a model for Table
   */
  class GTableModel extends AbstractTableModel implements  GedcomListener {

	private Vector gedcoms = new Vector();
	
	/**
	 * Empty Constructor
	 */
	GTableModel() {
	}

	/**
	 * Constructor with preset Gedcoms
	 */
	GTableModel(Vector pGedcoms) {

	  // Register as Listener
	  Gedcom gedcom;
	  for (int i=0;i<pGedcoms.size();i++) {
		gedcom = (Gedcom)pGedcoms.elementAt(i);
		gedcom.addListener(this);
		gedcoms.addElement(gedcom);
	  }

	  // Done
	}

	/**
	 * Returns number of Rows - that's the number of gedcoms
	 */
	public int getNumRows() {
	  return gedcoms.size();
	}

	/**
	 * Returns number of Columns for initialization
	 * Initially zero
	 */
	public int getNumColumns() {
	  return defaultHeaders.length;
	}

	/**
	 * Compares two given rows
	 */
	public int compareRows(int first, int second, int column) {
	  return 0;
	}

	/**
	 * Returns the Cell Object row,col - that's one of the gedcoms
	 */
	public Object getObjectAt(int row, int col) {

	  Gedcom gedcom = (Gedcom)gedcoms.elementAt(row);
	  String txt=null;

	  switch (col) {
	  case 0 :
		txt   = gedcom.getName();
		break;
	  case 1 :
	  case 2 :
	  case 3 :
	  case 4:
		txt   = ""+gedcom.getEntities(Gedcom.FIRST_ETYPE+col-1).getSize();
		break;
	  case 5 :
		txt   = "0";
		break;
	  case 6 :
		txt   = "0";
		break;
	  }

	  return txt;
	}

	/**
	 * Returns the Header Object col - that's one of the headline images
	 */
	public Object getHeaderAt(int col) {
	  return defaultHeaders[col];
	}

	/**
	 * Notification that a change in a Gedcom-object took place.
	 */
	public void handleChange(Change change) {

	  if (  (!change.isChanged(Change.EADD))
		  &&(!change.isChanged(Change.EDEL)) )
		return;

	  for (int i=0;i<gedcoms.size();i++) {
		if (gedcoms.elementAt(i)==change.getGedcom()) {
		  fireRowsChanged(new int[]{i});
		}
	  }

	}

	/**
	 * Notification that an entity has been selected.
	 */
	public void handleSelection(Selection selection) {
	}

	/**
	 * Notification that the gedcom is being closed
	 */
	public void handleClose(Gedcom which) {
		
	  // De-Register as listener
	  which.removeListener(this);

	  // Forget about it
	  gedcoms.removeElement(which);
	   
	  // Update GUI
	  fireNumRowsChanged();

	}

	/**
	 * Returns the Gedcom at given index
	 */
	public Gedcom getGedcom(int which) {
		return (Gedcom)gedcoms.elementAt(which);
	}
	
	/**
	 * Notification that a new gedcom has been opened
	 */
	public void addGedcom(Gedcom which) {

	  // Register as listener
	  which.addListener(this);

	  // Remember
	  gedcoms.addElement(which);

	  // Update GUI
	  fireNumRowsChanged();

	  // Select first
	  setSelectedRow(getNumRows()-1);

	  // Done
	}

	/**
	 * Returns the Gedcoms in this model
	 */
	Vector getGedcoms() {
		return gedcoms;
	}
  }
  /**
   * Constructor
   */
  public GedcomTable() {

	// Prepare a model
	model = new GTableModel();
	setModel(model);

	// Prepare rendering
	CellRenderer[] renderers = new CellRenderer[defaultHeaders.length];
	CellRenderer hrenderer = new GHeaderRenderer();
	for (int c=0;c<defaultHeaders.length;c++) {
	  renderers[c] = hrenderer;
	}
	setHeaderCellRenderers(renderers);

	// Done
  }      
/**
 * Add a Gedcom to this table
 */
public void addGedcom(Gedcom which) {
	model.addGedcom(which);
}
/**
 * Returns the Gedcom at given index
 */
public Gedcom getGedcom(int row) {
	return model.getGedcom(row);
}
/**
 * Returns the currently kept Gedcoms
 */
public Vector getGedcoms() {
	return model.getGedcoms();
}
  /**
   * Returns the actual selected Gedcom - or null if none selected
   */
  public Gedcom getSelectedGedcom() {
	int row = getSelectedRow();
	if (row<0)
	  return null;
	return model.getGedcom(row);
  }      
  /**
   * Sets a column width for this GedcomTable - null means default
   */
  public boolean setColumnWidths(int widths[]) {

	if (widths==null)
		widths = this.defaultWidths;

	return super.setColumnWidths(widths);
  }      
}