package genj.app;



import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import genj.gedcom.*;/**
 * Component for selecting from tree of TagPaths by clicking on CheckBoxes
 */
public class TagPathTree extends JScrollPane {

  private Vector  listeners = new Vector();
  private Gedcom  gedcom;
  private JTree   tree;
  private Node    root;

  private static final Color
	cSelection = new Color(160,160,255);

  /**
   * A Node in our TagPathSelector
   */
  static class Node implements TreeNode {

	String  tag;
	Vector  children;
	Node    parent;
	boolean selected;

	Node(String tag) {
	  this.tag = tag;
	  children = new Vector(4);
	}
	public void addChild(Node node) {
	  children.addElement(node);
	}
	public Node getChildFor(String tag) {
	  for (int i=0;i<children.size();i++) {
		Node child = (Node)children.elementAt(i);
		if (child.tag.equals(tag))
		  return child;
	  }
	  return null;
	}
	public TreeNode getParent() {
	  return parent;
	}
	public boolean getAllowsChildren() {
	  return true;
	}
	public TreeNode getChildAt(int index) {
	  return (Node)children.elementAt(index);
	}
	public int getChildCount() {
	  return children.size();
	}
	public int getIndex(TreeNode child) {
	  return children.indexOf(child);
	}
	public boolean isLeaf() {
	  return children.size()==0;
	}
	public Enumeration children() {
	  return children.elements();
	}

  }

  /**
   * Our Node Renderer
   */
  class NodeRenderer implements TreeCellRenderer {

	private JPanel    panel = new JPanel();
	private JCheckBox checkbox = new JCheckBox();
	private JLabel    label = new JLabel();

	NodeRenderer() {
	  panel.setLayout(new BorderLayout());
	  panel.add(checkbox,"West");
	  panel.add(label   ,"Center");

	  checkbox.setOpaque(false);
	  label.setOpaque(false);
	  panel.setOpaque(false);
	}

	public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {

	  Node node = (Node)value;

	  label.setText( node.tag );
	  label.setIcon( Property.calcDefaultImage(node.tag).getImageIcon() );
	  checkbox.setSelected(node.selected);

	  return panel;
	}
  }

  /**
   * Constructor
   */
  public TagPathTree() {

	// Prepare tree
	tree = new JTree(new DefaultTreeModel(new Node("")));
	tree.setShowsRootHandles(true);
	tree.setRootVisible(false);
	tree.setCellRenderer(new NodeRenderer());
	tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

	// Listening
	MouseAdapter madapter = new MouseAdapter() {
	  public void mouseClicked(MouseEvent me) {
		// Check wether some valid path has been clicked on
		TreePath path = tree.getPathForLocation(me.getX(),me.getY());
		if (path==null)
		  return;
		// Get Node and invert selection
		Node node = (Node)path.getLastPathComponent();
		node.selected = ! node.selected;
		// Signal to listeners
		fireSelectionChanged(path,node.selected);
		// Show it ... simple repaint ... but could be done better ... probably
		tree.repaint();
	  }
	};
	tree.addMouseListener(madapter);

	// Do a little bit of layouting
	setMinimumSize(new Dimension(160, 160));
	setPreferredSize(new Dimension(160, 160));
	getViewport().setView(tree);

	// Done
  }      
  /**
   * Adds another listener to this
   */
  public void addTagPathTreeListener(TagPathTreeListener listener) {
	listeners.addElement(listener);
  }      
  /**
   * Helper that creates nodes of given node from TagPaths
   */
  private void createNodesFor(Node root,TagPath path, int pos, boolean select) {

	// Is there a child with that tag already?
	Node child = root.getChildFor(path.get(pos));

	// Create a new Node for first tag of path
	if (child == null) {
	  child = new Node(path.get(pos));
	  root.addChild(child);
	}

	// Continue recursive?
	if (pos+1<path.length())
	  createNodesFor(child,path,pos+1,select);
	// .. or end of path?
	else
	  child.selected=select;

	// Done
  }      
  /**
   * Helper that expands all rows
   */
  private void expandRows() {
	for (int i=0;i<tree.getRowCount();i++)
	  tree.expandRow(i);
  }      
  /**
   * Signals a change in a node to the listeners
   * @param path the TreePath to the selected node
   */
  protected void fireSelectionChanged(TreePath path, boolean on) {

	// Loop through path elements (=Nodes)
	Object[] nodes = path.getPath();
	String s = "";

	// .. start with non-root as first
	for (int n=1;;n++) {
	  s+=((Node)nodes[n]).tag;

	  if (n+1==nodes.length)
		break;

	  s+=":";
	}

	// .. and create a TagPath
	TagPath p = new TagPath(s);

	// Go through listeners
	Enumeration enum = listeners.elements();
	while (enum.hasMoreElements()) {
	  ((TagPathTreeListener)enum.nextElement()).handleSelection(p,on);
	}
  }      
  /**
   * Removes one of ther listener to this
   */
  public void removeTagPathTreeListener(TagPathTreeListener listener) {
	listeners.removeElement(listener);
  }      
  /**
   * Sets the TagPaths to choose from
   */
  public void setPaths(TagPath[] paths) {

	// Create a new root of this tree
	root = new Node("Root");

	// Loop through used TagPaths
	for (int i=0;i<paths.length;i++) {
	  createNodesFor(root,paths[i],0,false);
	}

	tree.setModel(new DefaultTreeModel(root));

	// Show all rows
	expandRows();

	// Done
  }      
  /**
   * Returns the selection
   */
/*
  public TagPath[] getSelecton() {
	// hmm
	if (root==null)
	  return new TagPath[0];
	// Search for selections
  }
*/
  /**
   * Sets the selection
   */
  public void setSelection(TagPath[] paths) {

	// Loop through selected TagPaths
	for (int i=0;i<paths.length;i++) {
	  createNodesFor(root,paths[i],0,true);
	}

	tree.setModel(new DefaultTreeModel(root));

	// Show all rows
	expandRows();

	// Done
  }      
}