package genj.app;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.lang.reflect.Method;

import javax.swing.*;
import javax.swing.event.*;




import genj.gedcom.*;import genj.util.ImgIcon;/**
 * Component for selecting Tags by clicking on CheckBoxes in a list
 */
public class TagSelector extends JComponent {

  private JList  lChoose;
  private Vector vTags;
  private boolean isChanged = false;

  /**
   * Tag List Cell Renderer
   */
  class TagSelectorRenderer implements ListCellRenderer {

	private JPanel    panel = new JPanel();
	private JCheckBox check = new JCheckBox();
	private JLabel    label = new JLabel();

	/**
	 * Constructor
	 */
	TagSelectorRenderer() {
	  panel.setLayout(new BorderLayout());
	  panel.add(check,"West");
	  panel.add(label,"Center");

	  check.setOpaque(false);
	  label.setOpaque(false);
	  panel.setOpaque(false);
	}

	/**
	 * Return component for rendering list element
	 */
	public Component getListCellRendererComponent(JList list,Object value,int index,boolean isSelected,boolean cellHasFocus) {

	  Tag tag = (Tag)value;

	  label.setText( tag.tag );
	  label.setIcon( Property.calcDefaultImage(tag.tag).getImageIcon() );
	  check.setSelected( tag.selected );

	  return panel;
	}
  }
  
  /**
   * A tag to choose
   */
  private class Tag {
	Tag(String tag) { this.tag = tag; }
	String tag;
	boolean selected;
  }

  /**
   * Constructor
   */
  public TagSelector() {

	// Layout
	lChoose = new JList();
	lChoose.setCellRenderer(new TagSelectorRenderer());

	// Listening
	MouseAdapter madapter = new MouseAdapter() {
	  public void mouseClicked(MouseEvent me) {
		// Check wether some valid position has been clicked on
		int pos = lChoose.locationToIndex(me.getPoint());
		if (pos==-1)
		  return;
		// Get Node and invert selection
		Tag tag = (Tag)lChoose.getModel().getElementAt(pos);
		tag.selected = ! tag.selected;
		isChanged = true;
		// Show it ... simple repaint ... but could be done better ... probably
		lChoose.repaint();
	  }
	};
	lChoose.addMouseListener(madapter);


	setLayout(new BorderLayout());
	add(new JScrollPane(lChoose),"Center");

	// Done	
  }      
  /**
   * Returns the preferred size of this component
   */
  public Dimension getPreferredSize() {
	return new Dimension(64,64);
  }      
  /**
   * Returns selected tags
   */
  public String[] getSelectedTags() {

	// Loop through available tags
	Vector tags = new Vector(vTags.size());

	for (int i=0;i<vTags.size();i++) {
	  Tag tag = (Tag)vTags.elementAt(i);
	  if (tag.selected==true)
		tags.addElement(tag.tag);
	}

	String result[] = new String[tags.size()];
	tags.copyInto(result);

	return result;
  }      
  /**
   * Returns wether some selection change has occured after last setSelectedTags
   */
  public boolean isChanged() {
	return isChanged;
  }      
  /**
   * Specifies tags to be selected
   */
  public void selectTags(String[] tags) {

	isChanged = false;

	// Loop through available tags
  outer:
	for (int i=0;i<vTags.size();i++) {

	  // .. calculate Tag
	  Tag tag = (Tag)vTags.elementAt(i);

	  // .. check for selection
	  for (int j=0;j<tags.length;j++) {
		if (tags[j].equals(tag.tag)) {
		  tag.selected=true;
		  continue outer;
		}
	  }
	  tag.selected=false;

	  // .. next
	}

	// Done
	repaint();
  }      
  /**
   * Sets the tags used by this list
   */
  public void setTags(String[] tags) {

	vTags = new Vector(tags.length);
	for (int t=0;t<tags.length;t++)
	  vTags.addElement(new Tag(tags[t]));

	lChoose.setListData(vTags);

  }      
}