package genj.app;

import java.awt.*;
import javax.swing.*;

/**
 * Transaction - a transactional controller for a TransactionPanel
 */
public interface Transaction {

  public static final int
	NEXT   = 1,
	PREV   = 2,
	CANCEL = 4,
	OK     = 8,
	DONE   =16;

  /**
   * Ends this transaction - this action can't be ignored
   */
  public void cancel();      
  /**
   * Ends this transaction - this action can't be ignored by returning false
   */
  public boolean done();      
  /**
   * Returns the currently available actions for this transaction 
   * at current state
   */
  public int getActions();      
  /**
   * Returns an image that represents this transaction's state
   */
  public Image getImage();      
  /**
   * Returns the current visualization for this transaction
   */
  public JPanel getPanel();      
  /**
   * Moves one further in transaction - this action can be ignored by returning false
   */
  public boolean next();      
  /**
   * Initiates last step of this transaction - this action can be ignored by returning false
   */
  public boolean ok();      
  /**
   * Moves one back in transaction - this action can be ignored by returning false
   */
  public boolean prev();      
  /**
   * Set the TransactionPanel to use
   */
  public void setPanel(TransactionPanel set);      
  /**
   * Starts the transaction
   */
  public void start();      
}