package genj.applet;

import java.applet.*;
import java.awt.*; 
import java.awt.event.*; 
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.zip.ZipEntry;

 







import genj.tree.*;import genj.gedcom.*;import genj.util.*;import awtx.*;import genj.timeline.*;import genj.table.*;import genj.io.*;/**
 * GenJ Applet
 */
public class Applet extends java.applet.Applet implements Runnable {

	private final static int
		NOT_READY = -1,
		LOADING   =  0,
		ERROR     =  1,
		READY     =  2;
		
	private final static String[][] _views = {
		{ "Tree" ,   "images/NewTree.gif"     ,"genj.tree.TreeView"         ,"tree.1" },
		{ "Table",   "images/NewTable.gif"    ,"genj.table.TableView"       ,"table.1" },
		{ "Timeline","images/NewTimeline.gif" ,"genj.timeline.TimelineView" ,"timeline.1" }
	};

	private Hashtable views = new Hashtable();

	private int            state = NOT_READY;
	private Gedcom         gedcom;
	private Registry       registry;
	private Object         version;
	private GedcomReader   gedReader;

	private static final Resources resources = new Resources("genj.applet");

	/**
	 * A Link for a View
	 */
	private class ViewLink extends Component implements MouseListener {

		// Properties
		ImgIcon img;
		String title;
		String view;
		String key;
		
		// Constructor
		ViewLink(String sTitle, String sImage, String sView, String sKey) {
			img = new ImgIcon(this,sImage);
			title = sTitle;
			view = sView;
			key  = sKey ;
			setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			addMouseListener(this);
		}
		
		// Mouse
		public void mouseClicked(MouseEvent e) {
			openView(title,view,key);
		}
		public void mouseEntered(MouseEvent e) {
			getAppletContext().showStatus("Open "+title+" View");
		}
		public void mouseExited(MouseEvent e) {
			getAppletContext().showStatus("");
		}
		public void mouseReleased(MouseEvent e) {
			render(getGraphics(),false);
		}
		public void mousePressed(MouseEvent e) {
			render(getGraphics(),true);
		}
		
		// Painting
		public void paint(Graphics g) {
			render(g,false);
		}
		private void render(Graphics g, boolean pressed) {
			// Parms
			Dimension size = getSize();
			int w = img.getIconWidth(),
			    h = img.getIconHeight();
			FontMetrics fm = g.getFontMetrics();
			// Image
			g.drawImage(img.getImage(),(size.width-w)/2,size.height/2-h,null);
			// Text
			if (pressed)
				g.setColor(Color.red);
			else
				g.setColor(Color.black);
			w = fm.stringWidth(title);
			int x = (size.width-w)/2;
			int y = size.height/2+fm.getHeight()-fm.getDescent();
			g.drawString(title,x,y);
			// Link underline
			g.drawLine(x,y,x+w,y);
			// Done
		}
	}
  /**
   * Information about this applet.
   */
  public String getAppletInfo() {
	return resources.getString("applet.info")+"\nVersion: "+version;
  }      
  /**
   * Initializes the applet.
   */
  public void init() {
 
	// Don't need no re-init
	if (state!=NOT_READY)
	  return;

	// Look for Version information
	try {
	  version = Class.forName("GenJVersion").newInstance().toString();
	} catch (Exception e) {
	  version = "unknown";
	}

	// Report some information
	System.out.println(
	  resources.getString("applet.info")
	);
	System.out.println(
	  "Version: "+version
	);

	// Setup our container
	final Rootpane root = new Rootpane();
	root.setBackground(Color.white);
	root.setLayout(new BorderLayout());
	setLayout(new GridLayout(1,1));
	add(root);

	// Prepare Scala
	final Scala progress = new Scala();
	progress.setEditable(false);
	progress.setValue(0F);
	root.add(progress,"South");

	// Load Gedcom-Data
	state = LOADING;
	Thread t = new Thread(this);
	t.start();

	Runnable tracker = new Runnable() {
		public void run() {
			// Track the loading			
			while (state==LOADING) { try {
				Thread.sleep(100);
				if (gedReader!=null) {
					progress.setValue((float)gedReader.getProgress()/100);
					progress.setPrefix(gedReader.getState());
				}
			} catch (Exception e) {
			}}
			// Error?
			if (state!=READY)
				return;
			// Update display
			root.removeAll();
			root.setLayout(new GridLayout(1,1));
			for (int i=0;i<_views.length;i++) {
				ViewLink link = new ViewLink(
					_views[i][0],
					_views[i][1],
					_views[i][2],
					_views[i][3]
				);
				root.add(link);
			}
			root.validate();
			root.repaint();
			// Done
		}
	};
	t = new Thread(tracker);
	t.start();

	// Done
  }            
  /**
   * Opens a specified view
   */
  private void openView(String title,String view,String key) {

	// Already there?
	if (views.containsKey(key)) {
		Frame frame = (Frame)views.get(key);
		frame.show();
		frame.toFront();
		return;
	}
	  
	// Resolve class for viewing component
	Class c;
	try {
	  c = Class.forName(view);
	} catch (ClassNotFoundException e) {
	  throw new IllegalArgumentException("Couldnt find viewing class "+view);
	}

	Class ptypes[] = new Class[]{
	  Gedcom.class,
	  Registry.class,
	  Frame.class
	};

	java.lang.reflect.Constructor constructor;
	try {
	  constructor = c.getConstructor(ptypes);
	} catch (Exception e) {
	  throw new IllegalArgumentException("Couldnt find constructor for viewing class "+view);
	}

	// Create viewing component
	final Frame frame = new Frame(title);

	WindowAdapter wadapter = new WindowAdapter() {
	  public void windowClosing(WindowEvent e) {
		frame.setVisible(false);
	  }
	};
	frame.addWindowListener(wadapter);

	try {
	  Object params[] = new Object[]{
		gedcom,
		new Registry(registry,key),
		frame
	  };

	  frame.add( new Rootpane((Component) constructor.newInstance(params)) );
	} catch (Exception e) {
	  e.printStackTrace();
	  return;
	}

	// Done
	views.put(key,frame);
	
	frame.setSize(new Dimension(512,256));
	frame.show();
	
  }      
  /**
   * Our background loading thread
   */
  public void run() {

	String error =null;
	Origin origin=null;
	String sarchive  = getParameter("ZIP"    );
	String sgedcom   = getParameter("GEDCOM" );
	String detailUrl = getParameter("DETAIL");

	try {

		// .. Connect to Origin
		Origin.Connection connection;

		if (sarchive==null) {
			origin = new Origin(new URL(getDocumentBase(),sgedcom ));
			System.out.println(resources.getString("applet.loading",origin.toString()));
			connection = origin.open();
		} else {
			origin = new Origin(new URL(getDocumentBase(),sarchive));
			System.out.println(resources.getString("applet.loading",origin.toString()+"["+sgedcom+"]"));
			connection = origin.openFile(sgedcom);
		}

		// .. read gedcom from it
		gedReader = new GedcomReader(
			connection.getInputStream(),
			origin,
			connection.getLength()
		);
		gedcom=gedReader.readGedcom();

		// .. hopefully done
		System.out.println(resources.getString("applet.ready"));

	} catch (MalformedURLException ex) {
	  error=resources.getString("applet.malformed_url");
	  ex.printStackTrace();
	} catch (GedcomFormatException ex) {
	  error=resources.getString("applet.illegal_format")+" ("+ex.getLine()+"/"+ex.getMessage()+")";
	  ex.printStackTrace();
	} catch (Exception ex) {
	  error=resources.getString("applet.read_error");
	  ex.printStackTrace();
	} catch (Error er) {
	  er.printStackTrace();
	}

	// Error?
	if (error!=null) {
	  getAppletContext().showStatus(error);
	  state=ERROR;
	  return;
	}

	// Load Registry
	try {
		
		String sprop;
		if (origin.isZip()) {
			sprop = sgedcom+".properties";
			System.out.println(resources.getString("applet.loading",origin.toString()+"["+sprop+"]"));
		} else {
			sprop = origin.getName()+".properties";
			System.out.println(resources.getString("applet.loading",origin.toString()+".properties"));
		}
		Origin.Connection c = origin.openFile(sprop);
		registry = new Registry(c.getInputStream());

		// .. hopefully done
		System.out.println(resources.getString("applet.ready"));
	   
	} catch (IOException ex) {
	  ex.printStackTrace();
	  registry = new Registry();
	}

	// Hookup our DetailBridge
	if (detailUrl!=null) {
		System.out.println("Will show details for entities from "+detailUrl);
		gedcom.addListener(new DetailBridge(this,detailUrl));
	}
	
	
	// Done
	state=READY;
  }                  
}