package genj.edit;

import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;



import genj.gedcom.*;/**
 * Class which represents a javax.swing.tree.TreeModel for a property-tree
 */
public class PropertyTreeModel implements TreeModel {

  private Vector listeners = new Vector();
  private Property root;

  /**
   * Constructor
   */
  public PropertyTreeModel(Property root) {

	this.root = root;
  }      
  /**
   * Adds a listener to this model
   */
  public void addTreeModelListener(TreeModelListener l) {
	listeners.addElement(l);
  }      
  /**
   * Signals to listeners that properties have changed
   */
  public void firePropertiesChanged(Vector props) {

	// Do it for all changed properties
	Enumeration e = props.elements();
	while (e.hasMoreElements()) {

	  // .. use property
	  Property prop = (Property)e.nextElement();

	  // .. build event
	  Object path[] = root.getPathTo(prop);
	  if (path==null)
		continue;

	  TreeModelEvent ev = new TreeModelEvent(this,path);

	  // .. tell it to all listeners
	  Enumeration elisteners = listeners.elements();
	  while (elisteners.hasMoreElements()) {
		((TreeModelListener)elisteners.nextElement()).treeNodesChanged(ev);
	  }

	  // .. next changed property
	}
  }      
  /**
   * Signals to listeners that structure has changed
   */
  public void fireStructureChanged() {

	  Object path[] = { root };
	  TreeModelEvent ev = new TreeModelEvent(this,path);

	  // .. tell it to all listeners
	  Enumeration elisteners = listeners.elements();
	  while (elisteners.hasMoreElements()) {
		((TreeModelListener)elisteners.nextElement()).treeStructureChanged(ev);
	  }

  }      
  /**
   * Returns child by index of parent
   */
  public Object getChild(Object parent, int index) {

	Property prop = (Property)parent;
	return prop.getProperty(index);
	
  }      
  /**
   * Returns child count of parent
   */
  public int getChildCount(Object parent) {

	Property prop = (Property)parent;
	return prop.getNoOfProperties();
	
  }      
  /**
   * Returns index of given child from parent
   */
  public int getIndexOfChild(Object parent, Object child) {

	// Calculate index by fiven parent property
	int index = ((Property)parent).getIndexOf((Property)child);

	// This is zero-based
	return index-1;
	
  }      
  /**
   * Returns root of tree
   */
  public Object getRoot() {

	return root;
  }      
  /**
   * Tells wether object is a leaf
   */
  public boolean isLeaf(Object node) {

	Property prop = (Property)node;
	return prop.getNoOfProperties()==0;
	
  }      
  /**
   * Removes a Listener from this model
   */
  public void removeTreeModelListener(TreeModelListener l) {

	listeners.removeElement(l);
  }      
  /**
   * Changes a object at given path (not used here)
   */
  public void valueForPathChanged(TreePath path, Object newValue) {
  }      
}