package genj.gedcom;

import java.util.Vector;
import java.util.Enumeration;

/**
 * Class for encapsulating a set of Entities
 */
public class EntityList implements Cloneable {

  private Vector vector;

  /**
   * Class for running action on list of entities
   */
  public interface Action {
	/**
	 * Operate action on entity
	 * @return false if not to be continued
	 */
	public boolean on(Entity e);
  }

  /**
   * Constructor
   */
  public EntityList() {
	vector = new Vector();
  }      
  /**
   * Constructor with pre-occupied number of entities
   */
  public EntityList(int initialCapacity) {
	vector = new Vector(initialCapacity);
  }      
  /**
   * Constructor with number of pre-occupied entities
   * and number of persons to be incremented during growth
   */
  public EntityList(int initialCapacity, int capacityIncrement) {
	vector = new Vector(initialCapacity, capacityIncrement);
  }      
/**
 * Adds another entity to this set
 */
EntityList add(Entity entity) {
	vector.addElement(entity);
	return this;
}
/**
 * Adds other entities to this set
 */
EntityList add(EntityList entities) {
	vector.ensureCapacity(getSize()+entities.getSize());
	for (int i=0;i<entities.getSize();i++) {
	  vector.addElement(entities.get(i));
	}
	return this;
}
  /**
   * Sort set
   */
/*
  public void sort(Comparer comparer) {

	Object[] objs = new Object[vector.size()];
	vector.copyInto(objs);

	qsort(objs,0,objs.length-1,comparer);

	vector = new Vector(objs.length);
	for (int i=0;i<objs.length;i++)
	  vector.addElement(objs[i]);
  }
*/

  /**
   * Private qsort implementation
   */
/*
  private void qsort(Object objs[], int lo0, int hi0, Comparer c) {

	int lo = lo0;
	int hi = hi0;
	Entity mid;

	// Is max greater than min ?
	if (hi0<=lo0)
	  return;

	// Arbitrarily choose midpoint
	mid = (Entity) objs[ (lo0 + hi0) /2 ];

	// Loop through array until indices cross
	while (lo<=hi) {

	  // .. find first element greater or equal than partitioning element
	  while ( (lo<hi0) && (c.compare((Entity)objs[lo],mid)<0) )
		lo++;

	  // .. find element smaller or equal to partitioning element
	  while ( (hi>lo0) && (c.compare((Entity)objs[hi],mid)>0) )
		hi--;

	  // .. if the indexes have not crossed, swap
	  if (lo<=hi) {
		Entity e = (Entity)objs[lo];
		objs[lo] = (Entity)objs[hi];
		objs[hi] = e;
		++lo;
		--hi;
	  }

	  // next step
	}

	// If right index > left partition
	if (lo0<hi)
	  qsort(objs,lo0,hi,c);

	// If left index < right partition
	if (lo<hi0)
	  qsort(objs,lo,hi0,c);

	// Done
  }
*/

  /**
   * Interface for sorting criteria
   */
/*
  public interface Comparer {
	public int compare(Entity one, Entity two);
  }
*/

  /**
   * Clones this set
   */
  public Object clone() {

	EntityList result=null;
	try {
	  result = (EntityList)super.clone();
	  result.vector = (Vector)vector.clone();
	} catch (CloneNotSupportedException e) {
	}

	return result;
  }      
  /**
   * Checks wether an entity belongs to this set
   * @param entity the Entity to be looked for
   * @return true or false
   */
  public boolean contains(Entity entity) {
	if (entity==null)
	  return false;
	return vector.contains(entity);
  }      
/**
 * Copies kept Entities into Vector
 */
public void copyInto(Vector v) {
	Enumeration e = vector.elements();
	while (e.hasMoreElements())
	v.addElement(e.nextElement());
}
/**
 * Removes an entity
 */
EntityList del(int which) {
	vector.removeElementAt(which);
	return this;
}
/**
 * Removes an entity
 */
EntityList del(Entity which) {
	vector.removeElement(which);
	return this;
 }   
  /**
   * Returns one of the entities in this set
   */
  public Entity get(int which) {
	return ((Entity)vector.elementAt(which));
  }      
  /**
   * Returns entity as family
   */
  public Fam getFam(int which) {
	return ((Fam)get(which));
  }      
  /**
   * Returns entity as individual
   */
  public Indi getIndi(int which) {
	return ((Indi)get(which));
  }      
  /**
   * Returns entity as media
   */
  public Media getMedia(int which) {
	return ((Media)get(which));
  }      
  /**
   * Returns entity as note
   */
  public Note getNote(int which) {
	return ((Note)get(which));
  }      
  /**
   * Returns the number of entities in this set
   */
  public int getSize() {
	return(vector.size());
  }      
  /**
   * Operates on all entities in this list
   */
  public void operate(Action action) {
	Enumeration e = vector.elements();
	while (e.hasMoreElements()) {
	  if (!action.on((Entity)e.nextElement()))
		break;
	}
  }      
}