package genj.gedcom;

/** 
 * Class for encapsulating multimedia entry in a gedcom file
 */ 
public class Media extends PropertyMedia implements Entity {

	private String id = "";
	private Gedcom gedcom;

	private PropertyList foreignXRefs = new PropertyList();

  /**
   * Constructor for Multimedia
   */
  Media(Gedcom gedcom) throws GedcomException {
	
	// Call super's constructor
	super(null);

	// Entity
	this.gedcom = gedcom;
  }      
/**
 * Adds a PropertyForeignXRef to this entity
 */
public void addForeignXRef(PropertyForeignXRef fxref) {
	foreignXRefs.add(fxref, this);
}
  /**
   * Notification to entity that it has been added to a Gedcom
   */
  public void addNotify(Gedcom gedcom) {
	this.gedcom = gedcom;
  }      
  /**
   * Notification to entity that it has been deleted from a Gedcom
   */
  public void delNotify() {
	
	// Notify to properties
	super.delNotify();

	// Remove all foreign XRefs
	foreignXRefs.deleteAll();
	
	// Break connection
	this.gedcom = null;
  }      
/**
 * Removes a property 
 * This overrides the default behaviour by first
 * looking in this entity's foreign list
 */
public boolean delProperty(Property which) {

	if (foreignXRefs.contains(which)) {
		foreignXRefs.delete(which);
		return true;
	}
	return super.delProperty(which);
}
  /**
   * Gedcom this entity's in
   * @return containing Gedcom
   */
  public Gedcom getGedcom() {
	return gedcom;
  }      
  /**
   * Returns the id of this entity
   */
  public String getId() {
	return id; 
  }      
  /**
   * Returns the main property of this entity
   */
  public Property getProperty() {
	return this;
  }      
  /**
   * Returns the type to which this entity belongs
   * INDIVIDUALS, FAMILIES, MULTIMEDIAS, NOTES, ...
   */
  public int getType() {
	return Gedcom.MULTIMEDIAS;
  }      
  /**
   * Set Gedcom this entity's in
   */
  public void setGedcom(Gedcom gedcom) {
	this.gedcom=gedcom;
  }      
  /**
   * Sets entity's id.
   * @param id new id
   */
  public void setId(String id) {
	this.id=id;

	// Done
  }      
}