package genj.gedcom;

import java.util.*; 
import java.lang.reflect.Method;
import java.lang.reflect.Constructor;

 
import genj.util.*;/**
 * Base Class for all kinds of Gedcom-Properties
 */ 
public abstract class Property {

  private static final Property NO_KNOWN_PROPERTIES[] = {};
  private static final String   NO_KNOWN_SUB_TAGS  [] = {};
  private static final String   EVT_SUB_TAGS       [] = new String[]{ "DATE", "PLAC", "ADDR", "AGNC", "TYPE" };
  private static final String   IAT_SUB_TAGS       [] = new String[]{ "ADDR", "AGNC", "DATE", "PLAC", "TYPE" };
   
  private Property  parent=null;
  private PropertyList properties=null;

  public static final int UP   = 1,
						  DOWN = 2;

  public final static int NO_MULTI=1, MULTI_NEWLINE=2, MULTI_BLOCK=3;

  private static Hashtable tag2defs;
  private static PropertyDefinition unknownDefinition = 
		new PropertyDefinition(null,null,new String[]{});

  protected static final PropertyDefinition propDefs[] = {
		
		// FAM Record
		new PropertyDefinition("FAM","Fam",new String[]{"MARR","CHIL","HUSB","WIFE","NOTE","ANUL","CENS","DIV","DIVF","ENGA","MARB","MARC","MARL","MARS"}) ,
		new PropertyDefinition("CHIL","Child") ,
		new PropertyDefinition("HUSB","Husband") ,
 		new PropertyDefinition("WIFE","Wife") ,

		// INDI Record
		new PropertyDefinition("INDI","Indi",new String[]{"NAME","SEX","BIRT","DEAT","FAMS","FAMC","RESI","GRAD","OCCU","OBJE","NOTE","CAST","DSCR","EDUC","IDNO","NATI","NCHI","PROP","RELI","SSN","TITL","CHR","BURI","CREM","ADOP","BAPM","BAPL","BARM","BASM","BLES","CHRA","CONF","FCOM","ORDN","NATU","EMIG","IMMI","CENS","PROB","WILL","RETI"}) ,
		new PropertyDefinition("NAME","Name") , 
		new PropertyDefinition("SEX" ,"Sex") ,
		new PropertyDefinition("FAMC" ,"FamilyChild") ,
 		new PropertyDefinition("FAMS" ,"FamilySpouse") ,

		// OBJE Record
		new PropertyDefinition("OBJE","Indi") ,
		new PropertyDefinition("BLOB","Blob") ,

		// NOTE Record
		new PropertyDefinition("NOTE","Note", new String[]{"DATE"}) ,

		// INDI Attributes
		new PropertyDefinition("CAST","IndividualAttribute",IAT_SUB_TAGS) ,
		new PropertyDefinition("DSCR","IndividualAttribute",IAT_SUB_TAGS) ,
		new PropertyDefinition("EDUC","IndividualAttribute",IAT_SUB_TAGS) ,
		new PropertyDefinition("IDNO","IndividualAttribute",IAT_SUB_TAGS) ,
		new PropertyDefinition("NATI","IndividualAttribute",IAT_SUB_TAGS) ,
		new PropertyDefinition("NCHI","IndividualAttribute",IAT_SUB_TAGS) ,
		new PropertyDefinition("OCCU","IndividualAttribute",IAT_SUB_TAGS) ,
		new PropertyDefinition("PROP","IndividualAttribute",IAT_SUB_TAGS) ,
		new PropertyDefinition("RELI","IndividualAttribute",IAT_SUB_TAGS) ,
		new PropertyDefinition("RESI","IndividualAttribute",IAT_SUB_TAGS) ,
		new PropertyDefinition("SSN" ,"IndividualAttribute",IAT_SUB_TAGS) ,
		new PropertyDefinition("TITL","IndividualAttribute",IAT_SUB_TAGS) ,

		// INDI Events
		new PropertyDefinition("BIRT","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("CHR" ,"Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("DEAT","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("BURI","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("CREM","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("ADOP","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("BAPM","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("BAPL","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("BARM","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("BASM","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("BLES","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("CHRA","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("CONF","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("FCOM","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("ORDN","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("NATU","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("EMIG","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("IMMI","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("CENS","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("PROB","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("WILL","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("GRAD","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("RETI","Event",EVT_SUB_TAGS) ,

		// FAM Events
		new PropertyDefinition("ANUL","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("CENS","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("DIV" ,"Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("DIVF","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("ENGA","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("MARR","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("MARB","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("MARC","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("MARL","Event",EVT_SUB_TAGS) ,
		new PropertyDefinition("MARS","Event",EVT_SUB_TAGS) ,
		
		// Individual Attributes
		new PropertyDefinition("ADDR","Address",new String[]{ "CITY", "POST" }) ,
		new PropertyDefinition("CONT","Continuation") ,
		new PropertyDefinition("DATE","Date") ,

		new PropertyDefinition("AGE" ,"Age") ,
		new PropertyDefinition("CAUS","Cause") ,
		new PropertyDefinition("CITY","City") ,
		new PropertyDefinition("PLAC","Place",new String[]{ "CONT" }) ,
		new PropertyDefinition("POST","PostalCode") ,

		// Generic Attributes
		new PropertyDefinition("AGNC","GenericAttribute") ,
		new PropertyDefinition("TYPE","GenericAttribute") ,
		new PropertyDefinition("TITL","GenericAttribute") ,
		new PropertyDefinition("FORM","GenericAttribute") ,
		new PropertyDefinition("OBJE","Media"           ,new String[]{"TITL","FORM", "BLOB", "FILE"}) ,
		new PropertyDefinition("FILE","File"            )
  };	

  /**
   * Initializer
   */
  static {

	// Create a Hashtable for lookups	  
	tag2defs = new Hashtable(propDefs.length);

	// .. fill it up with property classes
	for (int i=0;i<propDefs.length;i++) {
	  PropertyDefinition propDef = propDefs[i];
	  tag2defs.put(propDef.getTag(),propDef);
	}

	// .. done

  }

  /**
   * Interface for an Iterator that enables the interested to
   * iterate through multiple lines of data
   */
  public interface LineIterator {
	/**
	 * Returns wether this iterator has more lines
	 */
	public boolean hasMoreValues();
	/**
	 * Returns the next line of this iterator
	 */
	public String getNextValue() throws NoSuchElementException;
  }

  /**
   * Helper for getPathTo
   */
  private Vector _getPathTo(Property prop) {

	// Prop is me ?
	if (prop==this) {
	  Vector result = new Vector(10);
	  result.addElement(this);
	  return result;
	}

	// One of my properties ?
	Vector result = null;
	for (int i=0;i<getNoOfProperties();i++) {
	  // .. test it
	  result = getProperty(i)._getPathTo(prop);
	  if (result!=null)
		break;
	  // .. next
	}

	// Found ? Add myself
	if (result!=null)
	  result.addElement(this);

	// Done
	return result;
  }      
/**
 * Adds default properties to this property
 */
public void addDefaultProperties() {
}    
/**
 * Method for notifying being added to another property
 */
public void addNotify(Property parent) {
	this.parent=parent;
	noteAddedProperty();
}
/**
 * Adds another property to this property
 * @param prop new property to add
 */
public final void addProperty(Property prop) {

	// Make sure we have a children's list
	if (properties==null)
		properties=new PropertyList();

	// Remember 
	properties.add(prop,this);

	// We have to remember this as a new known (sub-)property
	getPropertyDefinition(getTag()).addSubTag(prop.getTag());

	// Done
}

  /**
   * Calculates a property's standard image from given TAG
   */
  public static ImgIcon calcDefaultImage(String tag) {

	// Find class for tag
	Class c = getPropertyDefinition(tag).getPropertyClass();

	// Calculate image
	try {
	  Method method = c.getMethod("getDefaultImage",new Class[0]);
	  return (ImgIcon)method.invoke(null,new Object[0]);
	} catch (Exception e) {
	  System.out.println(e.getMessage());
	}	
	// Error means unknown
	return PropertyUnknown.getDefaultImage();
  }            
  /**
   * Calculates a property's standard proxy from given TagPath
   */
  public static String calcDefaultProxy(TagPath path) {

	// Find class for tag
	Class c = getPropertyDefinition(path.getLast()).getPropertyClass();
	
	// Get proxy value
	Class  argtypes[] = { path.getClass() };
	Object arg     [] = { path            };
	try {
	  Method method = c.getMethod("getProxy",argtypes);
	  return (String)method.invoke(null,arg);
	} catch (Exception e) {
	  System.out.println(e.getMessage());
	}	
	// Error means unknown
	return PropertyUnknown.getProxy(null);
  }          
  /**
   * Compares this property to another property
   * @return -1 this < property <BR>
   *          0 this = property <BR>
   *          1 this > property
   */
  public int compareTo(Property p) {
	return getValue().compareTo(p.getValue());
  }      
  /**
   * Create a property object from a TAG, VALUE (can do preferred
   * sub-properties, too)
   */
  public static Property createInstance(String tag,String value) {

	// Find class for tag
	Class c = getPropertyDefinition(tag).getPropertyClass();
	
	// Instantiate Property object
	Constructor constructor = null;
	Object      object = null;

	try {
	  // .. get constructor of property
	  Object parms[] = { tag, value };
	  Class  parmclasses[] = { parms[0].getClass() , parms[1].getClass()};
	  constructor = c.getConstructor(parmclasses);

	  // .. get object
	  object = constructor.newInstance(parms);

	  // Done		
	  return (Property)object;

	} catch (Exception e) {
	  e.printStackTrace();
	  System.out.println(e);
	  System.out.println(e.getMessage());
	}

	// Error means unknown
	return new PropertyUnknown(tag,value);
  }          
/**
 * Removes all properties
 */
public final void delAllProperties() {
	if (properties!=null)
		properties.deleteAll();  
	properties=null;
}
/**
 * Method for notifying being removed from parent property
 */
public void delNotify() {

	// Remember it
	noteDeletedProperty();

	// Say it to properties
	if (properties!=null)
		properties.deleteAll();

	// Done
}
  /**
   * Removes an array of properties
   */
  public void delProperties(Property[] which) {
	for (int i=0;i<which.length;i++)
	  delProperty(which[i]);
  }      
/**
 * Removes a property by looking in the property's properties
 * list and eventually calling delProperty recursively
 */
public boolean delProperty(Property which) {

	// No properties ?
	if (properties == null)
	  return false;

	// Look for first class properties
	if (properties.contains(which)) {
		properties.delete(which);
		return true;
	}
	
	// Look for second class properties
	for (int i=0;i<properties.getSize();i++) {
	  if ( properties.get(i).delProperty(which) )
		return true;
	}
	
	// Not found
	return false;	
}
  /**
   * Returns the default image which is associated with this property.
   */
  public static ImgIcon getDefaultImage() {
	return PropertyUnknown.getDefaultImage();
  }      
  /**
   * Returns a warning string that describes what happens when this 
   * property would be deleted
   * @return warning as <code>String</code>, <code>null</code> when no warning
   */  
  public String getDeleteVeto() {
	return null;
  }      
  /**
   * Calculates the max. depth of properties this property has.
   */
  public int getDepthOfProperties() {
  
	if (properties==null)
	  return 0;
	  
	int result = 0;
	for (int i=0;i<properties.getSize();i++) {
	  result = Math.max( result, properties.get(i).getDepthOfProperties()+1 );
	}
		
	// Done
	return result;
  }      
  /**
   * Returns the entity this property belongs to
   */  
  public Entity getEntity() {
  
	// Parent == Entity ?
	if (this instanceof Entity)
	  return (Entity)this;

	// No Parent ?
	if (parent==null)
	  return null;

	// Ask Parent
	return parent.getEntity();
  }      
  /**
   * Returns the gedcom this property belongs to
   */  
  public Gedcom getGedcom() {

	// Entity ?
	Entity entity = getEntity();
	if (entity==null)
	  return null;
  
	// Ask Entity
	return entity.getGedcom();
  }      
  /**
   * Returns the image which is associated with this property.
   */
  public abstract ImgIcon getImage(boolean checkValid);      
  /**
   * Returns the index of given property - or 0 when not found.
   * @param prop Property to look for
   */
  public int getIndexOf(Property prop) {

	// Look through properties
	for (int i=0;i<getNoOfProperties();i++) {
	  if (getProperty(i)==prop)
		return i;
	}

	// Not found
	return 0;
  }      
  /**
   * Returns some explanationary information about this property.
   */
  public String getInfo() {

	// Find Info that matches tag
	String tag = getTag();

	String name = Gedcom.getResources().getString(tag+".name");
	String info = Gedcom.getResources().getString(tag+".info");

	return name+":\n"+info;
  }      
/**
 * Returns a list of (sub-)properties which are known 
 */
public final Property[] getKnownProperties() {

	// Find the definition for this property
	PropertyDefinition propDef = (PropertyDefinition)tag2defs.get(getTag());
	Vector tags = propDef.getSubTags();
		
//	if ((hash==null)||(hash.isEmpty()))
//		return NO_KNOWN_PROPERTIES;

	// Build a result
	Property result[] = new Property[tags.size()];
	int p = 0;
	Enumeration e = tags.elements();
	while (e.hasMoreElements()) {
		result[p++] = createInstance(
			(String)e.nextElement(),
			""
		);
	}

	// Done
	return result;
	
}      
  /**
   * Returns a LineIterator which can be used to iterate through
   * several lines of data in case of isMultiLine equals true;
   * Should be implemented by properties with several lines.
   * Returns null when single line only
   */
  public LineIterator getLineIterator() {
	return null;
  }      
  /**
   * Returns the next sibling of this property
   * @return property beside or null
   */
  public Property getNextSibling() {
	
	// No parent ?
	if (parent==null)
	  return null;

	// Wich index is this one ?
	int index = parent.getIndexOf(this);
	if (index==-1)
	  return null;

	// Me the last ?
	if (index==parent.getNoOfProperties()-1)
	  return null;

	// Return next sibling
	return parent.getProperty(index+1);
  }      
  /**
   * Calculates the number of properties this property has.
   */
  public int getNoOfProperties() {
	if (properties==null)
	  return 0;
	return properties.getSize();
  }      
  /**
   * Calculates the number of properties this property has.
   * When recursive is true, sub-properties are counted recursively, too.
   * When valid is true, only valid sub-properties are counted.
   */
  public int getNoOfProperties(boolean recursive, boolean validOnly) {
  
	if (properties==null)
	  return 0;
	  
	int result = 0;
	for (int i=0;i<properties.getSize();i++) {
	  if (properties.get(i).isValid() || (!validOnly))
		result ++;
	  if (recursive)
		result += properties.get(i).getNoOfProperties(true,validOnly);
	}

	return result;
  }      
  /**
   * Returns the property this property belongs to
   */  
  public Property getParent() {
	return parent;
  }      
  /**
   * Returns path of properties to specified property
   */
  public Property[] getPathTo(Property prop) {

	// Try to find vector
	Vector v = _getPathTo(prop);
	if (v==null)
	  return null;

	// Transform to array
	Property[] result = new Property[v.size()];
	for (int i=0;i<result.length;i++)
	  result[result.length-1-i] = (Property)v.elementAt(i);

	// Done
	return result;
  }      
  /**
   * Returns the previous sibling of this property
   * @return property beside or null
   */
  public Property getPreviousSibling() {

	// No parent ?
	if (parent==null)
	  return null;

	// Wich index is this one ?
	int index = parent.getIndexOf(this);
	if (index==-1)
	  return null;

	// Me the first ?
	if (index==0)
	  return null;

	// Return previous sibling
	return parent.getProperty(index-1);
  }      
  /**
   * Returns this property's properties
   */
  public PropertyList getProperties() {
	if (properties==null)
	  properties = new PropertyList();
	return properties;
  }        
  /**
   * Returns this property's properties by path
   */
  public Property[] getProperties(TagPath path, boolean validOnly) {

	// No properties there ?
	if (properties==null)
	  return new Property[0];

	// Gather 'em
	Vector vresult = new Vector(properties.getSize());
	getPropertiesInto(path,vresult,validOnly);
	
	Property[] result = new Property[vresult.size()];
	for (int i=0;i<result.length;i++)
	  result[i]=(Property)vresult.elementAt(i);

	// Done
	return result;
  }          
  /**
   * Helper for getProperties
   */
  private void getPropertiesInto(TagPath path,Vector fill, boolean validOnly) {

	// Correct here ?
	if (!path.getNext().equals(getTag())) {
	  path.back();
	  return;
	}

	// Me ?
	if (!path.hasMore()) {
	  path.back();
	  // .. only when valid
	  if ( (validOnly) && (!isValid()) ) {
		return;
	  }
	  // .. add me
	  fill.addElement(this);
	  // .. done
	  return;
	}
		
	// Does this one have properties ?
	if (properties==null) {
	  path.back();
	  return;
	}
	  
	// Search in properties
	Property p;
	for (int i=0;i<properties.getSize();i++) {
	  properties.get(i).getPropertiesInto(path,fill,validOnly);
	}
	
	path.back();    
  }      
  /**
   * Returns this property's nth property
   */
  public Property getProperty(int n) {
	return properties.get(n);
  }      
/**
 * Returns this property's property by path
 */
public Property getProperty(TagPath path, boolean validOnly) {

	String next;

	while (true) {	
		// Empty TagPath means us
		if (!path.hasMore()) {
			// .. validity?
			if ( (validOnly) && (!isValid()) )
				return null;
			// .. we're o.k.
			return this;
		}

		// This is what we are looking for
		next = path.getNext();

		// Ignore 1st in case of entity
		if (!((this instanceof Entity)&&(getTag().equals(next))))
			break;
	}
	
	// Loop through the children
	Property child;
	Property result;
	PropertyList children = getProperties();
	for (int i=0;i<children.getSize();i++) {
		
		child = children.get(i);
	 
		// .. check that child 
		if (child.getTag().equals(next)) {
			result = child.getProperty(path, validOnly);
			if (result != null)
				return result;
		}

		// .. we have to keep on looking
	}

	// Getting here ... restore path for a sibling
	path.back();    

	return null;
}          
/**
 * Looks up a PropDef for given tag
 */
private static PropertyDefinition getPropertyDefinition(String tag) {
	PropertyDefinition propDef = (PropertyDefinition)tag2defs.get(tag);
	if (propDef==null) {
		propDef = unknownDefinition;
	}
	return propDef;
}
  /**
   * Returns the logical name of the proxy-object which knows this object
   */
  public String getProxy() {
	return "Unknown";
  }      
  /**
   * Returns the logical name of the proxy-object which knows this object
   */
  public static String getProxy(TagPath path) {
	return "Unknown";
  }      
  /**
   * Returns the Gedcom-Tag of this property
   */
  public abstract String getTag();      
  /**
   * Returns the value of this property as string.
   */
  abstract public String getValue();      
  /**
   * Returns wether this property consists of several lines
   * @return NO_MULTI, MULTI_NEWLINE, MULTI_BLOCK
   * @see getLineIterator()
   */
  public int isMultiLine() {
	return NO_MULTI;
  }      
  /**
   * Returns <b>true</b> if this property is valid
   */
  public boolean isValid() { 
	return true; 
  }      
  /**
   * Moves one of the (sub)properties up/down
   */
  public boolean moveProperty(Property which, int how) { 

	// Does this one have properties ?
	if (properties==null)
	  return false;

	// Search in properties
	Property p;
	for (int i=0;i<properties.getSize();i++) {
	  p = properties.get(i);
	  // ... move it
	  if (p==which) {

		int j = i+(how==UP?-1:1);
		try {
		  properties.swap(i,j);

		  getGedcom().setUnsavedChanges(true);
		} catch (Exception e) {
		  return false;
		}
		return true;
	  }
	  // ... maybe in property ?
	  if (p.moveProperty(which,how))
		return true;
	  // ... try next
	}
	
	// Not moved ! (?)
	return false;
  }      
  /**
   * Notify Gedcom that this property has been added
   */
  protected void noteAddedProperty() {
	Gedcom gedcom = getGedcom();
	if (gedcom!=null)
	  gedcom.noteAddedProperty(this);
  }      
  /**
   * Notify Gedcom that this property has been deleted
   */
  protected void noteDeletedProperty() {
	Gedcom gedcom = getGedcom();
	if (gedcom!=null)
	  gedcom.noteDeletedProperty(this);
  }      
  /**
   * Notify Gedcom that this property has been changed
   */
  protected void noteModifiedProperty() {
	Gedcom gedcom = getGedcom();
	if (gedcom!=null)
	  gedcom.noteModifiedProperty(this);
  }      
  /**
   * Sets this property's value as string.
   */
  public abstract boolean setValue(String newValue);      
/**
 * The default toString returns the value of this property
 * NM 19990715 introduced to allow access to a property on a
 *             more abstract level than getValue()
 */
public String toString() {
	if (!(this instanceof Entity))
		return getTag()+" "+getValue();
	Entity e = (Entity)this;
	return "@"+e.getId()+"@ "+getTag()+" "+getValue();
}      
}