package genj.gedcom;

import java.util.*;


import genj.util.*;/** 
 * Gedcom property which constitutes an Address 
 */ 
public class PropertyAddress extends Property {

	private String address;
	private ImgIcon image;

  /**
   * Member class for iterating through adress' lines
   */
  private class AddressLineIterator implements Property.LineIterator {

	private StringTokenizer tokens;
	
	/**
	 * Constructor which inits tokens
	 */
	AddressLineIterator() {
	  tokens = new StringTokenizer(address,"\n");
	}

	/**
	 * Returns wether this iterator has more lines
	 */
	public boolean hasMoreValues() {
	  return tokens.hasMoreTokens();
	}
	
	/**
	 * Returns the next line of this iterator
	 */
	public String getNextValue() throws NoSuchElementException {
	  return tokens.nextToken();
	}
	
  }
  
/**
 * Constructor of address Gedcom-line 
 */
public PropertyAddress() {
	this("");
}  
  /**
   * Constructor of address Gedcom-line
   * @param address initial value
   */
  public PropertyAddress(String address) {
	this.address=address;
  }      
  /**
   * Constructor with Tag,Value,Level parameters
   * @param tag Tag
   * @param value Value
   * @param level Level
   */
  public PropertyAddress(String tag, String value) {
	setValue(value);
  }      
  /**
   * Adds all known properties to this property 
   */
  public void addDefaultProperties() {
	noteModifiedProperty();
	addProperty(new PropertyCity("")); 
	addProperty(new PropertyPostalCode(""));
  }        
  /**
   * Returns the default image which is associated with this property.
   * @return image
   */
  public static ImgIcon getDefaultImage() {
	return Images.imgPlace;
  }      
  /**
   * Returns the image which is associated with this property.
   * @return image
   */
  public ImgIcon getImage(boolean checkValid) {
	return getDefaultImage();
  }      
  /**
   * Returns a LineIterator which can be used to iterate through
   * several lines of this address
   */
  public LineIterator getLineIterator() {

	return new AddressLineIterator();
	
  }      
  /**
   * Returns the logical name of the proxy-object which knows this object
   * @return proxy's logical name
   */
  public String getProxy() {
	return "MLE";
  }      
  /**
   * Returns the name of the proxy-object which knows properties looked
   * up by TagPath
   * @return proxy's logical name
   */
  public static String getProxy(TagPath path) {
	return "MLE"; 
  }      
  /**
   * Returns the Gedcom-Tag of this property
   * @return tag as <code>String</code>
   */
  public String getTag() {
	return "ADDR";
  }      
  public String getValue() {

	// More than one line ?
	int pos = address.indexOf('\n');
	if (pos<0)
	  return address;
	// Value
	return address.substring(0,pos)+"...";
  }      
  /**
   * Yes, this property incorporates several lines with newlines
   */
  public int isMultiLine() {
	return MULTI_NEWLINE;
  }      
  public boolean setValue(String value) {
	noteModifiedProperty();
	this.address=value;
	return true;
  }      
}