package genj.gedcom;

import java.util.*;

 
import genj.util.*;/**
 * Gedcom property which constitutes a Cause
 */
public class PropertyCause extends Property {

  private String cause;
  
  /**
   * Member class for iterating through cause's lines
   */
  private class CauseLineIterator implements Property.LineIterator {

	private StringTokenizer tokens;
	
	/**
	 * Constructor which inits tokens
	 */
	CauseLineIterator() {
	  tokens = new StringTokenizer(cause,"\n");
	}

	/**
	 * Returns wether this iterator has more lines
	 */
	public boolean hasMoreValues() {
	  return tokens.hasMoreTokens();
	}
	
	/**
	 * Returns the next line of this iterator
	 */
	public String getNextValue() throws NoSuchElementException {
	  return tokens.nextToken();
	}
	
  }
/**
 * Constructor of cause Gedcom-line
 */
public PropertyCause(String cause) {
	this.cause=cause;
}  
  /**
   * Constructor of cause Gedcom-line
   */
  public PropertyCause(String tag, String value) {
	setValue(value);
  }      
  public static ImgIcon getDefaultImage() {
	return Images.imgCause;
  }      
  public ImgIcon getImage(boolean checkValid) {
	return getDefaultImage();
  }      
  /**
   * Returns a LineIterator which can be used to iterate through
   * several lines of this cause
   */
  public LineIterator getLineIterator() {

	return new CauseLineIterator();
	
  }      
  public String getProxy() {
	return "MLE";
  }      
  /**
   * Returns the name of the proxy-object which knows properties looked
   * up by TagPath
   * @return proxy's logical name
   */
  public static String getProxy(TagPath path) {
	return "MLE"; 
  }      
  public String getTag() {
	return "CAUS";
  }      
  public String getValue() {

	// More than one line ?
	int pos = cause.indexOf('\n');
	if (pos<0)
	  return cause;
	// Value
	return cause.substring(0,pos)+"...";
  }      
  /**
   * Yes, this property incorporates several lines
   */
  public int isMultiLine() {
	return MULTI_NEWLINE;
  }      
  public boolean setValue(String value) {
	noteModifiedProperty();
	cause=value;
	return true;
  }      
}