package genj.gedcom;

import java.util.*;

 
import genj.util.*;/**
 * Gedcom property which constitutes an Event
 */
public class PropertyEvent extends Property {


	private String tag;

/**
 * Constructor of address Gedcom-line
 */
public PropertyEvent(String tag) {
	
	// Remember
	this.tag = tag;

	// Done
}  
  /**
   * Constructor of address Gedcom-line
   */
  public PropertyEvent(String tag, String value) {
	
	// Remember
	this.tag = tag;

	// Done
  }        
  /**
   * Adds all default properties to this property
   */
  public void addDefaultProperties() {
	addProperty(new PropertyDate()); 
	addProperty(new PropertyPlace("")); 
  }        
  /**
   * Returns the date of the event
   */
  public PropertyDate getDate() {

	return getDate(true);

  }      
  /**
   * Returns the date of the event
   * @param valid specifies wether data has to be valid to be found
   */
  public PropertyDate getDate(boolean valid) {

	// Try to get date-property which is valid
//	Property prop = getProperty(new TagPath(getTag()+":DATE"),valid);
	Property prop = getProperty(new TagPath("DATE"),valid);
	if (prop==null)
	  return null;

	// Return as Date
	return (PropertyDate)prop;
  }        
  /**
   * Calculate event's date
   */
  public String getDateAsString() {
	// Calculate properties of event
	PropertyList props = getProperties();
	if (props == null)	
	  return "";

	// look for property DATE
	Property prop;
	
	for (int i=0;i<props.getSize();i++) {
	  prop = props.get(i);
	  if ( prop instanceof PropertyDate )
		return ((PropertyDate)prop).toString();
	}	

	// No information
	return "";
  }      
  /**
   * Returns the default image of this property
   */
  public static ImgIcon getDefaultImage() {
	return Images.imgEvent;
  }      
  /**
   * Returns the image of this property
   */
  public ImgIcon getImage(boolean checkValid) {

	// Try to find image for Tag	
	if (tag.equals("BIRT")) return Images.imgBirth;
	if (tag.equals("DEAT")) return Images.imgDeath;
	if (tag.equals("BURI")) return Images.imgBurial;
	if (tag.equals("MARR")) return Images.imgMarriage;
	if (tag.equals("BARM")) return Images.imgBarBat;
	if (tag.equals("BASM")) return Images.imgBarBat;
	if (tag.equals("IMMI")) return Images.imgMigration;
	if (tag.equals("EMIG")) return Images.imgMigration;
 
	// Return Default
	return Images.imgEvent;
  }      
  /**
   * Returns the logical name of the proxy-object which knows this object
   */
  public String getProxy() { 
	return "Empty";
  }      
  /**
   * Returns the name of the proxy-object which knows properties looked
   * up by TagPath
   * @return proxy's logical name
   */
  public static String getProxy(TagPath path) {
	return "Empty"; 
  }      
  
  /**
   * Returns the tag of this property
   */
  public String getTag() {
	return tag;
  }      
  /**
   * Returns the value of this property
   */
  public String getValue() {
	return "";
  }      
  
  /**
   * Sets the value of this property
   */
  public boolean setValue(String value) {
	return false;
  }      
	private static String[] tags;/**
 * Returns the list of tags which identify PropertyEvents
 */
public static String[] getTags() {

	// Already calculated?
	if (tags!=null)
		return tags;

	Vector v = new Vector(100);	
	for (int i=0;i<propDefs.length;i++) {
		PropertyDefinition def = propDefs[i];
		if (def.isEvent())
			v.addElement(def.getTag());
	}

	String[] result = new String[v.size()];
	Enumeration e = v.elements();
	for (int i=0;e.hasMoreElements();i++)
		result[i] = e.nextElement().toString();

	tags = result;
	return result;
}}