package genj.gedcom;


import java.awt.*;
import java.io.*;
import java.net.*;

import genj.util.*;/**
 * Gedcom property which constitutes a link to an external file
 */
public class PropertyFile extends Property {

  private String  file;
  private boolean isIconChecked = false;
  private ImgIcon valueAsIcon   = null;
  
  /**
   * Constructor of FILE Gedcom-line
   */
  public PropertyFile() {
  }      
  /**
   * Constructor of FILE Gedcom-line
   */
  public PropertyFile(String file) {
	this.file=file;
  }      
  /**
   * Constructor of FILE Gedcom-line
   */
  public PropertyFile(String tag, String value) {
	setValue(value);
  }      
  /**
   * Returns the default Image for this Property
   */
  public static ImgIcon getDefaultImage() {
	return Images.imgDisk;
  }      
  public ImgIcon getImage(boolean checkValid) {
	return getDefaultImage();
  }      
  /**
   * Returns the logical name of the proxy-object which knows this object
   */
  public String getProxy() {
	return "File";
  }      
  /**
   * Returns the name of the proxy-object which knows properties looked
   * up by TagPath
   * @return proxy's logical name
   */
  public static String getProxy(TagPath path) {
	return "File";
  }      
  /**
   * Returns the tag of this property
   */
  public String getTag() {
	return "FILE";
  }      
/**
 * Returns this property's value
 */
public String getValue() {

	if (file==null)
		return "";
		
	if (!isRelativeChecked) {
		String relative = getGedcom().getOrigin().calcRelativeLocation(file);
		if (relative !=null)
			file = relative;
		isRelativeChecked = true;
	}
	return file;
}      
  /**
   * Tries to return the data of the referenced file as an icon
   */
  public synchronized ImgIcon getValueAsIcon() {

	// Already calculated?
	if (isIconChecked) {
	  return valueAsIcon;
	}
	isIconChecked = true;
	valueAsIcon   = null;
	
	// Check File for Image ?
	if ((file==null)||(file.trim().length()==0))
	  return null;

	// Open InputStream
	try {
	  valueAsIcon = new ImgIcon(getInputStream());
	} catch (IOException e) {
	}

	// Done
	return valueAsIcon;

  }        
/**
 * Sets this property's value
 */
public boolean setValue(File f) {
	return setValue("file:"+f);
}          
/**
 * Sets this property's value
 */
public boolean setValue(String value) {
	
	// Remember the change
	noteModifiedProperty();

	// Remember the value
	file=value.replace('\\','/');

	// Reinit our icon calculation
	isIconChecked = false;
	isRelativeChecked = false;
	return true;
}        
  private boolean isRelativeChecked = false;public InputStream getInputStream() throws IOException {
	return getGedcom().getOrigin().openFile(file).getInputStream();
}}