package genj.gedcom;

import java.util.*; 

 
import genj.util.*;/**
 * Class for encapsulating a property that describes a family-husband-relationship
 */
public class PropertyHusband extends PropertyXRef {

  /**
   * Constructor with reference
   */
  public PropertyHusband(PropertyXRef target) {
	super(target);
  }      
  /**
   * Constructor with Tag,Value parameters
   */
  public PropertyHusband(String tag, String value) {
	super(tag,value);
  }      
  public static ImgIcon getDefaultImage() {
	return Images.imgLink2Indi;
  }      
  /**
   * Returns a warning string that describes what happens when this 
   * property would be deleted
   * @return warning as <code>String</code>, <code>null</code> when no warning
   */  
  public String getDeleteVeto() {
	return "The connection to the referenced husband and its reference to this family are lost";
  }      
  /**
   * Returns the husband
   */
  public Indi getHusband() {
	return (Indi)getReferencedEntity();
  }      
  /**
   * Returns the image which is associated with this property.
   */
  public ImgIcon getImage(boolean checkValid) {
	if (checkValid&&(getReferencedEntity()==null))
	  return Images.imgLinkXIndi;
	return Images.imgLink2Indi;
  }      
  /**
   * Returns the Gedcom-Tag of this property
   */
  public String getTag() {
	return "HUSB";
  }      
  /**
   * Links reference to entity (if not already done)
   * @exception GedcomException when property has no parent property,
   * or a double husband/wife situation would be the result
   */
  public void link() throws GedcomException {
	
	// Something to do ?
	if (getHusband()!=null)
	  return;

	// Get enclosing family ?
	Fam fam;
	try {
	  fam = (Fam)getEntity();
	} catch (ClassCastException ex) {
	  throw new GedcomException("HUSB can't be linked to individual when not in family");
	}

	// Prepare some VARs
	Property p;
	Property ps[];
	
	// Enclosing family has a husband already ?
	if (fam.getHusband()!=null)
	  throw new GedcomException("Family @"+fam.getId()+"@ can't have two husbands");

	// Look for husband (not-existing -> Gedcom throws Exception)
	String id = getReferencedId();
	Indi husband = getGedcom().getIndiFromId(id);

	if (husband==null)
	  throw new GedcomException("Couldn't find husband with ID "+id);

	// Enclosing family has indi as child or wife ?
	if (fam.getWife()==husband)
	  throw new GedcomException("Individual @"+id+"@ is already wife in family @"+fam.getId()+"@");

	Indi children[] = fam.getChildren();
	for (int i=0;i<children.length;i++) {
	  if ( children[i] == husband )
		throw new GedcomException("Individual @"+id+"@ is already child in family @"+fam.getId()+"@");
	}

	// Connect back from husband (maybe using invalid back reference)
	ps = husband.getProperties(new TagPath("INDI:FAMS"),false);
	PropertyFamilySpouse pfs;
	for (int i=0;i<ps.length;i++) {
	  pfs = (PropertyFamilySpouse)ps[i];
	  if ( (!pfs.isValid()) && (pfs.getReferencedId().equals(fam.getId())) ) {
		pfs.setTarget(this); // Changed Oct 23 from pfs.setTarget(pfs);
		setTarget(pfs);      // Inserted Oct 23
		return;
	  }
	}
	
	// .. new back referencing property
	pfs = new PropertyFamilySpouse(this);
	husband.addProperty(pfs);
	setTarget(pfs);
	
	// Done
  }      
  /**
   * Returns a verbose description of this property
   */
  public String toString() {
	return "Husband "+getValue();
  }      
}