package genj.gedcom;


 
import genj.util.*;/** 
 * Class for encapsulating a media as property
 */ 
public class PropertyMedia extends PropertyXRef {

  /**
   * Constructor with reference
   * @param entity reference of entity this property links to
   */
  public PropertyMedia(PropertyXRef target) {
	super(target);
  }      
  /**
   * Constructor with Tag,Value parameters
   * @param tag property's tag
   * @param value property's value
   */
  public PropertyMedia(String tag, String value) {
	super(tag,value);
  }      
  /**
   * Adds all default properties to this property
   */
  public void addDefaultProperties() {

	noteModifiedProperty();
	
	// Just add 'em
	addProperty(new PropertyGenericAttribute("TITL"));
	addProperty(new PropertyGenericAttribute("FORM"));

	// Am I an entity ?
	if (getEntity()==this) {
	  addProperty(new PropertyBlob());
	} else {
	  addProperty(new PropertyFile());
	}

	// Done
  }            
  /**
   * Returns the (default) image of this property
   */
  public static ImgIcon getDefaultImage() {
	return Images.imgMedia;
  }      
  /**
   * Returns the image of this property
   */
  public ImgIcon getImage(boolean checkValid) {
	return getDefaultImage();
  }      
  /**
   * Returns the logical name of the proxy-object which knows this object
   */
  public String getProxy() {
	// Entity Media ?
	if (this instanceof Entity)
	  return "Entity";
	// OBJE linked to Entity Media or FILE
	return "XRef";
  }      
  /**
   * Returns the name of the proxy-object which knows properties looked
   * up by TagPath
   * @return proxy's logical name
   */
  public static String getProxy(TagPath path) {
	if (path.length()>1) {
	  return "XRef";
	}
	return "Entity";
  }      
  /**
   * Returns the tag of this property
   */
  public String getTag() { 
	return "OBJE";
  }      
  /**
   * Links reference to entity (if not already done)
   * @exception GedcomException when processing link would result in inconsistent state
   */
  public void link() throws GedcomException {

	// Get enclosing entity ?
	Entity entity = getEntity();
	// .. Me Media-Property or -Entity?
	if (this==entity)
	  return;  // outa here
	
	// Something to do ?
	if (getReferencedEntity()!=null)
	  return;

	// Look for media
	String id = getReferencedId();
	if (id.length()==0)
	  return;
	
	Media media = getGedcom().getMediaFromId(id);
	if (media==null)
		throw new GedcomException("Couldn't find entity with ID "+id);

	// Create a back-reference
	PropertyForeignXRef fxref = new PropertyForeignXRef(this);
	media.addForeignXRef(fxref);

	// .. and point to it
	setTarget(fxref);

	// Are there any properties that can be deleted ?
	Property[] known = getKnownProperties();
	for (int i=0;i<known.length;i++) {
	  TagPath path = new TagPath(getTag()+":"+known[i].getTag());
	  Property[] props = getProperties(path,false);
	  delProperties(props);
	}
	
	// Done
	
  }      
}