package genj.gedcom;

import java.util.Vector; 

 
import genj.util.*;/**
   Gedcom property which constitutes a Name
 */
public class PropertyName extends Property {

  private String lastName  = null,
				 firstName = null;

  private String nameAsString;

  /**
   * Constructor for Name Gedcom line
   */
  public PropertyName() {
	// Setup data
	lastName  = "";
	firstName = "";
  }      
  /**
   * Constructor for Name Gedcom line
   */
  public PropertyName(String tag, String value) {
	// Setup data
	setValue(value);
  }      
  /**
   * Compares this property to another property
   * @return -1 this < property <BR>
   *          0 this = property <BR>
   *          1 this > property
   */
  public int compareTo(Property p) {

	if (!(p instanceof PropertyName))
	  return 0;

	PropertyName pn = (PropertyName)p;

	return getName().compareTo(pn.getName());
  }      
  public static ImgIcon getDefaultImage() {
	return Images.imgName;
  }      
  public String getFirstName() {
	return firstName;
  }      
  public ImgIcon getImage(boolean checkValid) {
	if (checkValid&&(nameAsString!=null))
	  return Images.imgError;
	return Images.imgName;
  }      
  /**
   * Returns localized label for first name
   */
  static public String getLabelForFirstName() {
	return Gedcom.getResources().getString("prop.name.firstname");
  }      
  /**
   * Returns localized label for last name
   */
  static public String getLabelForLastName() {
	return Gedcom.getResources().getString("prop.name.lastname");
  }      
  public String getLastName() {
	return lastName;
  }      
  public String getName() {
	if (nameAsString!=null)
	  return nameAsString;
	if (firstName.length()==0)
	  return lastName;
	return lastName + ", " + firstName;
  }      
  public String getProxy() {
	if (nameAsString!=null)
	  return "Unknown";
	return "Name";
  }      
  /**
   * Returns the name of the proxy-object which knows properties looked
   * up by TagPath
   * @return proxy's logical name
   */
  public static String getProxy(TagPath path) {
	return "Name"; 
  }      
  public String getTag() {
	return "NAME";
  }      
  public String getValue() {
	if (nameAsString != null)
	  return nameAsString;
	if (lastName.equals(""))
	  return firstName;
	return firstName+" /"+lastName+"/";
  }      
  /**
   * Sets name to a new value
   */
  public void setName(String first, String last) {

	noteModifiedProperty();
	
	// Make sure no Information is kept in base class
	nameAsString=null;;
  
	lastName  = last.trim();
	firstName = first.trim();

	// Done
  }      
  public boolean setValue(String newValue) {

	noteModifiedProperty();
	
	// New empty Value ?
	if (newValue==null) {
	  nameAsString=null;
	  lastName=null;
	  firstName=null;
	  
	  // Done
	  return true;
	}
	
	// Only name specified ?
	if (newValue.indexOf('/') == -1) {
	  nameAsString=null;
	  firstName = newValue;
	  lastName = "";
	  
	  // Done
	  return true;
	}

	// Name AND First name
	String f = newValue.substring( 0 , newValue.indexOf('/') ).trim();
	String l = newValue.substring( newValue.indexOf('/') + 1 );
	
	// ... wrong format (2 x '/'s !)  
	if (l.indexOf('/') == -1)  {
	  nameAsString=newValue;
	  // Done
	  return false;
	}	

	// ... format ok
	l = l.substring( 0 , l.indexOf('/') );
	
	// Done
	nameAsString=null;
	firstName = f;
	lastName  = l;
	// Done
	return true;
  }      
}