package genj.gedcom;

import java.util.Vector; 
import java.awt.event.*; 

 
import genj.util.*;/**
 * Class for Gedcom Sex Property
 */
public class PropertySex extends Property {

  private int sex;
  private String sexAsString;

  /**
   * Constructor for Gedcom Sex Line
   */
  public PropertySex() { 
	// Super's constructor 
	super(); 
	// Setup data
	sex = Gedcom.MALE;
	// Done	 
  }      
  /**
   * Constructor for Gedcom Sex Line
   */
  public PropertySex(int setSex) {
	// Super's constructor
	super();
	// Setup data
	if ((setSex != Gedcom.MALE) && (setSex != Gedcom.FEMALE) )
	  throw new IllegalArgumentException("Wrong sex value for constructor"+setSex);

	sex = setSex;		
	// Done	
  }      
  /**
   * Constructor for Gedcom Sex Line
   */
  public PropertySex(String tag, String value) {
	// Setup data
	if (value.length() == 0)
	  setValue("M");
	else
	  setValue(value);
	// Done	
  }      
  public static ImgIcon getDefaultImage() {
	return Images.imgMale;
  }      
  public static ImgIcon getDefaultImage(int sex) {
	switch (sex) {
	case Gedcom.MALE:
	  return Images.imgMale;
	case Gedcom.FEMALE:
	  return Images.imgFemale;
	}
	throw new IllegalArgumentException("Unknown sex");
  }      
  public ImgIcon getImage(boolean checkValid) {
	if (checkValid&&(sexAsString!=null))
	  return Images.imgError;
	if (sex == Gedcom.MALE)
	  return Images.imgMale;
	return Images.imgFemale;
  }      
  /**
   * Returns localized label for sex
   */
  static public String getLabelForSex() {
	return Gedcom.getResources().getString("prop.sex");
  }      
  /**
   * Returns localized label for sex of male/female
   */
  static public String getLabelForSex(int which) {

	if (which==Gedcom.MALE)
	  return Gedcom.getResources().getString("prop.sex.male");
	return Gedcom.getResources().getString("prop.sex.female");
  }      
  /**
   * Returns the logical proxy to render/edit this property
   */
  public String getProxy() {
	if (sexAsString!=null)
	  return "Unknown";
	return "Sex";
  }      
  /**
   * Returns the name of the proxy-object which knows properties looked
   * up by TagPath
   * @return proxy's logical name
   */
  public static String getProxy(TagPath path) {
	return "Sex"; 
  }      
  public int getSex() {
	return sex;
  }      
  public String getTag() {
	return "SEX";
  }      
  public String getValue() {
	if (sexAsString != null)
	  return sexAsString;
	if (sex == Gedcom.MALE)
	  return "M";
	return "F";
  }      
  /**
   * Returns <b>true</b> if this property is valid
   */
  public boolean isValid() { 
	return (sexAsString==null);
  }      
  public void setSex(int newSex) {
	noteModifiedProperty();
	if ((newSex != Gedcom.MALE) && (newSex != Gedcom.FEMALE) ) {
	  throw new IllegalArgumentException("Wrong sex value "+newSex);
	}	
	sex = newSex;
	// Done
  }      
  public boolean setValue(String newValue) {
	
	noteModifiedProperty();
	
	// No information ?
	if (newValue.length()!=1) {
	  sexAsString=newValue;
	  // Done
	  return false;
	}	
	// Female or Male ?
	switch (newValue.charAt(0)) {
	  case 'f' :
	  case 'F' :
		sex = Gedcom.FEMALE;
		sexAsString=null;
		return true;
	  case 'm' :
	  case 'M' : sex = Gedcom.MALE;
		sexAsString=null;
		// Done
		return true;
	}
	// Done
	sexAsString=newValue;
	// Done
	return false;
  }      
}