package genj.option;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.lang.reflect.Method;

import javax.swing.*;
import javax.swing.event.*;




import genj.gedcom.*;import genj.util.ImgIcon;/**
 * Option panel for choosing a property from a list of properties
 */
public class OptionChooseProperty extends JComponent implements ItemListener, ListSelectionListener {

  private JRadioButton rbChoose,rbNew;
  private JTextField tfNew;
  private JList lChoose;
  private JScrollPane spInfo;
  private JTextPane tpInfo;

  /**
   * Tag List Cell Renderer
   */
  class TagListRenderer extends JLabel
	implements ListCellRenderer {

	boolean isSelected;
		
	/**
	  Return component for rendering list element
	*/
	public Component getListCellRendererComponent(JList list,Object value,int index,boolean isSelected,boolean cellHasFocus) {
	  Property prop = (Property)value;
	  setText(prop.getTag());
	  try { 
		// .. get class of property
		ImgIcon img   = prop.getImage(false);
		setIcon(new ImageIcon( img.getImage() ));
	  } catch (Exception e) { 
		System.out.println("Unexpected error while retrieving property's image!"); 
		setIcon(new ImageIcon( PropertyUnknown.getDefaultImage().getImage() ));
	  }	 

	  this.isSelected = isSelected;
	  return this;
	}

	/**
	  * paint is subclassed to draw the background correctly.  JLabel
	  * currently does not allow backgrounds other than white, and it
	  * will also fill behind the icon.  Something that isn't desirable.
	  */
	public void paint(Graphics g) {
	  Color            bColor;

	  if (isSelected)
		bColor = Color.yellow;
	  else
		bColor = (getParent() != null) ? getParent().getBackground() : getBackground();

	  g.setColor(bColor);
	  g.fillRect(0 , 0, getWidth()-1, getHeight()-1);
	  
	  super.paint(g);
	}
  }  
  

  /**
   * Constructor for Option
   */
  public OptionChooseProperty(Property[] knownProps) {

	// Layout
	GridBagLayout layout = new GridBagLayout();
	setLayout(layout); 

	// Checkbox for known props
	rbChoose = new JRadioButton("Known tags",knownProps.length>0); 
	rbChoose.setEnabled(knownProps.length>0); 
	rbChoose.addItemListener(this);
	rbChoose.setAlignmentX(0);
	add(rbChoose,1,1,2,1,false); 
	
	// .. List of tags 
	lChoose = new JList(knownProps); 
	lChoose.setEnabled(knownProps.length>0); 
	lChoose.setCellRenderer(new TagListRenderer());
	if (knownProps.length==0) lChoose.setPrototypeCellValue(new PropertyUnknown("XXXXX",""));
	lChoose.addListSelectionListener(this);
	JScrollPane sp = new JScrollPane(lChoose);
	add(sp,1,2,1,1,true); 
	 
	// .. Info field 
	tpInfo = new JTextPane(); 
	tpInfo.setText("");
	tpInfo.setEditable(false); 
	spInfo = new JScrollPane(tpInfo,JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,JScrollPane.HORIZONTAL_SCROLLBAR_NEVER) {
		public Dimension getPreferredSize() {
		  return new Dimension(256,128);
		}
	  };
	add(spInfo,2,2,1,1,true); 

	// RadioButton for new props
	rbNew = new JRadioButton("New tag",knownProps.length==0); 
	rbNew.addItemListener(this);
	rbNew.setAlignmentX(0);
	add(rbNew,1,3,2,1,false); 

	ButtonGroup group = new ButtonGroup();
	group.add(rbChoose);
	group.add(rbNew);
				  
	// Create Lower Part 
	tfNew = new JTextField();
	tfNew.setEnabled(knownProps.length==0);
	tfNew.setAlignmentX(0);
	add(tfNew,1,4,2,1,false);

	// Focus
	if (knownProps.length==0)
	  tfNew.requestFocus();
	else
	  lChoose.requestFocus();
		  
  }        
  /**
   * Helper method that adds components to a container with gridbaglayout
   */
  private void add(Component c,int x,int y,int w,int h,boolean grow) {
	add(c);
	
	GridBagConstraints s = new GridBagConstraints();
	s.gridx = x;
	s.gridy = y;
	s.gridwidth = w;
	s.gridheight= h;
	s.weightx = grow ? 1 : 0;
	s.weighty = grow ? 1 : 0;
	s.fill = grow ? GridBagConstraints.BOTH : GridBagConstraints.HORIZONTAL;
	
	((GridBagLayout)getLayout()).setConstraints(c,s);
  }      
  /**
   * Returns the resulting properties
   */
  public Property[] getResultingProperties() {  

	Property[] result = null;
	
	// ... prepare list of selected properties
	if (rbChoose.isSelected() == true) {
	  Object[] objs = lChoose.getSelectedValues();
	  if ( (objs==null) || (objs.length==0) )
		result = null;
	  else {
		result = new Property[objs.length];
		for (int i=0;i<objs.length;i++)
		  result[i] = (Property)objs[i];
	  }
	} else { 
	  // ... create a single entry property list
	  if (tfNew.getText().equals(""))
		result = null;
	  else {
		result = new Property[1];
		result[0] = Property.createInstance(tfNew.getText(),"");
	  }
	}	 
	
	return result;
  }      
  /**
   * RadioButtons have been changed
   */
  public void itemStateChanged(ItemEvent e)	{ 
	if (e.getSource() == rbChoose) { 
	  lChoose.setEnabled(true); 
	  tfNew.setEnabled(false);		 
	  lChoose.requestFocus(); 
	}	 
	if (e.getSource() == rbNew) { 
	  lChoose.clearSelection();
	  lChoose.setEnabled(false); 
	  tfNew.setEnabled(true);		 
	  tfNew.requestFocus(); 
	}	 
  }      
  /**
   * One of the tag-items in the item list has been (de-)selected
   */
  public void valueChanged(ListSelectionEvent e) {

	// Check selection
	Object[] selection = lChoose.getSelectedValues();

	// None selected
	if ((selection==null)||(selection.length==0)) {
	  tpInfo.setText("");
	  return;
	}

	// Show info of last selected
	Property prop=(Property)selection[selection.length-1];
	tpInfo.setText(prop.getInfo());
	if (!rbChoose.isSelected())
	  rbChoose.doClick();
	
	// Done
  }      
}