package genj.option;

import java.awt.*; 
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;




import genj.gedcom.*;import genj.util.ImgIcon;/**
 * Class for deleting a new entity visually
 */
public class OptionDelEntity extends JPanel implements ActionListener, GedcomListener {

  private Entity entity   = null;
  private Vector gedcoms  = null;

  private JFrame       frame;
  private JLabel       lWhich;
  private JButton      bDelete;

  static private ImageIcon imgTrash;
  
  /**
   * Constructor for new indiviual option
   */
  public OptionDelEntity(JFrame frame, Vector gedcoms, Gedcom start) {

	// Images ?
	if (imgTrash==null) {
	  imgTrash = new ImageIcon(new ImgIcon(this,"Trash.gif").getImage());
	}

	// Data
	this.frame=frame;
	this.gedcoms=gedcoms;

	// Panel for Which
	lWhich = new JLabel(OptionResources.getString("mask.none"),SwingConstants.CENTER) {
	  public Dimension getPreferredSize() {
		return new Dimension(128,64);
	  }
	};
	
	// Panel for Action
	JPanel pAction = new JPanel();
	pAction.setLayout(new FlowLayout(FlowLayout.RIGHT));
	
	bDelete = new JButton(OptionResources.getString("mask.delete"));
	pAction.add(bDelete);

	// Layout
	setLayout(new BorderLayout(3,3));
	add(lWhich,"Center");
	add(pAction,"South");
	add(new JLabel(imgTrash),"West");

	// Listeners
	Enumeration gs = gedcoms.elements();
	while (gs.hasMoreElements()) {
		((Gedcom)gs.nextElement()).addListener(this);
	}
	bDelete.addActionListener(this);
	
	// Initial relative
	if (start==null)
	  setEntity(null);
	else {
	  setEntity(start.getLastEntity());
	}
	
	// Done
  }      
  /**
	Gets triggered when a button has been pressed
  */
  public void actionPerformed(ActionEvent e) {

	// Delete
	if (entity==null)
	  return;

	Gedcom gedcom = entity.getGedcom();
	if (gedcom==null)
	  return;

	if (!gedcom.startTransaction())
		return;
	
	try {	
		gedcom.deleteEntity(entity);
	} catch (GedcomException ge) {
	}
	gedcom.endTransaction();
	
	// Done & Dispose (?)
	frame.dispose();
  }      
  /**
   * Helper function that adds components to panels with GridBagLayout
   */
  private void addComponent(Container to,GridBagLayout layout,Component which,int gridx,int gridy,int gridwidth,int gridheight,boolean fill) {
	GridBagConstraints c = new GridBagConstraints();
	c.gridx=gridx;
	c.gridy=gridy;
	c.gridheight=gridheight;
	c.gridwidth=gridwidth;
	c.fill = fill ? GridBagConstraints.BOTH : GridBagConstraints.NONE;
	layout.setConstraints(which,c);
	to.add(which);
  }      
  /**
   * Notification that a change in a Gedcom-object took place.
   */
  public void handleChange(Change change) {
	// Back off in case someone deleted entities
	if ( change.isChanged(Change.EDEL) )
	  setEntity(null);
	// Done
  }      
  /**
   * Notification that the gedcom is being closed
   */
  public void handleClose(Gedcom which) {
	if ((entity!=null)&&(entity.getGedcom()==which))
	  setEntity(null);
	which.removeListener(this);
  }      
  /**
   * Notification that an entity has been selected.
   */
  public void handleSelection(Selection selection) {
	setEntity(selection.getEntity());
  }      
  /**
   * Notification when component is not used any more
   */
  public void removeNotify() {
	super.removeNotify();
	
	Enumeration gs = gedcoms.elements();
	while (gs.hasMoreElements()) {
		((Gedcom)gs.nextElement()).removeListener(this);
	}
  }      
  /**
   * Remembers entity to be deleted
   * @param entity reference to be remembered
   */
  private void setEntity(Entity entity) {

	// No entity any more ?
	if (entity==null) {

	  lWhich.setText( OptionResources.getString("mask.none") );
	  lWhich.setIcon( null );
	  bDelete.setEnabled(false);

	} else {

	  lWhich.setText( "@"+entity.getId()+"@" );
	  lWhich.setIcon( new ImageIcon(entity.getProperty().getImage(false).getImage()) );
	  bDelete.setEnabled(true);

	}

	// Remember
	this.entity=entity;

	// Done
  }      
}