package genj.report;

import genj.gedcom.Gedcom;
import java.io.*;
import javax.swing.*;

/**
 * Interface of a user definable GenjJ Report
 */
public interface Report {

  /**
   * Returns the author of this script
   */
  public String getAuthor();        
  /**
   * Returns information about this report. A report
   * has to override this method to return a <code>String</code>
   * containing information about the author, version
   * and copyright of the report.
   * @return a string containing information about the author, version
   * and copyright of the report
   */
  public String getInfo();        
  /**
   * Returns the name of this report - should be localized.
   */
  public String getName();        
  /**
   * Tells wether this report doesn't change information in the Gedcom-file
   */
  public boolean isReadOnly();        
  /**
   * Called by GenJ to start this report's execution - has to be
   * overriden by a user defined report.
   * @param reportView report component initiating the report execution
   * @param gedcom gedcom object on which the report is supposed
   * @param out object for writing output
   * @exception InterruptedException in case running Thread is interrupted
   */
  public boolean start(ReportView reportView, Gedcom gedcom, ReportWriter out) throws InterruptedException;                
  /**
   * Returns true if this report uses STDOUT
   */
  public boolean usesStandardOut();        
}
