package genj.report;

import javax.swing.*; 
import javax.swing.tree.*; 
import javax.swing.event.*; 
import javax.swing.border.EmptyBorder;
 
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.*;

import genj.gedcom.*;
import genj.util.*;
import genj.app.EntitySelector;
 
/** 
 * Component for running reports on genealogic data
 */
public class ReportView extends JPanel implements ListSelectionListener, ActionListener {

  private Gedcom gedcom;
  private Frame frame;
  private JLabel lAuthor;
  private JTextPane tpInfo;
  private JScrollPane spOutput;
  private JTextArea taOutput;
  private JList listOfReports;
  private JTabbedPane tabbedPane;
  private JButton bStart,bStop,bClose,bSave,bReload;
  private Thread thread;
  private boolean isInterrupted;
  private static ReportLoader loader;
  private static ImageIcon imgShell,imgGui;

  /**
   * Report Renderer
   */
  class ReportRenderer extends JLabel implements ListCellRenderer {

	/**
	 * Return component for rendering list element
	 */
	public Component getListCellRendererComponent(JList list,Object value,int index,boolean isSelected,boolean cellHasFocus) {
	  
	  Report report = (Report)value;
	  setText(report.getName());
	  if (report.usesStandardOut())
		setIcon(imgShell);
	  else
		setIcon(imgGui);

	  if (isSelected)
		setBackground(list.getSelectionBackground());
	  else
		setBackground(list.getBackground());

	  return this;
	}

	/**
	  * paint is subclassed to draw the background correctly.  JLabel
	  * currently does not allow backgrounds other than white, and it
	  * will also fill behind the icon.  Something that isn't desirable.
	  */
	public void paint(Graphics g) {
	  Color            bColor;

	  g.setColor(getBackground());
	  g.fillRect(0 , 0, getWidth()-1, getHeight()-1);
	  
	  super.paint(g);
	}
  }  

  /**
   * Constructor
   */
  public ReportView(Gedcom gedcom,Registry view,Frame frame) {

	// Inherited
	super();

	// Data
	this.gedcom=gedcom;
	this.frame =frame ;

	imgShell = new ImageIcon(new ImgIcon(this,"ReportShell.gif").getImage());
	imgGui   = new ImageIcon(new ImgIcon(this,"ReportGui.gif"  ).getImage());
	
	// Look for reports
	loadReports(false);

	// Layout for this component
	setLayout(new BorderLayout());
	
	// Noteboook in Center
	tabbedPane = new JTabbedPane();
	add(tabbedPane,"Center");
	
	// Panel for Report
	JPanel reportPanel = new JPanel();
	reportPanel.setBorder(new EmptyBorder(3,3,3,3));
	GridBagHelper reportGridBag = new GridBagHelper(reportPanel);
	tabbedPane.add("Reports",reportPanel);

	// ... List of reports
	Report reports[] = loader.getReports();
	listOfReports = new JList(reports);
	listOfReports.setCellRenderer(new ReportRenderer());
	listOfReports.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	listOfReports.addListSelectionListener(this);

	JScrollPane spList = new JScrollPane(listOfReports);
	reportGridBag.add(spList,1,1,1,4,GridBagHelper.GROW_BOTH);

	// ... Report's author
	lAuthor = new JLabel("Unknown");
	lAuthor.setForeground(Color.black);
	
	reportGridBag.add(new JLabel("Author"),2,1,2,1,0);
	reportGridBag.add(new JLabel(" "     ),2,2,1,1,0);
	reportGridBag.add(lAuthor             ,3,2,1,1,0);
	
	// ... Report's infos
	tpInfo = new JTextPane(); 
	tpInfo.setEnabled(false);
	JScrollPane spInfo = new JScrollPane(tpInfo,JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,JScrollPane.HORIZONTAL_SCROLLBAR_NEVER) {
	  public Dimension getPreferredSize() {
		return new Dimension(256,128);
	  }
	};
	
	reportGridBag.add(new JLabel("Information"),2,3,2,1,0                      );
	reportGridBag.add(spInfo                   ,3,4,1,1,GridBagHelper.GROW_BOTH);

	// Panel for Report Output
	taOutput = new JTextArea();
	taOutput.setEditable(false);
	
	spOutput = new JScrollPane(taOutput,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS) {
	  public Dimension getPreferredSize() {
		return new Dimension(256,128);
	  }
	};
	tabbedPane.add("Output",spOutput);
	
	// Buttons at bottom
	JPanel buttonPanel = new JPanel();
	add(buttonPanel,"South");
	
	buttonPanel.setLayout(new BoxLayout(buttonPanel,BoxLayout.X_AXIS));
	buttonPanel.add(   bStart = newButton("Start"  ,"Start selected report","START"  ,true ) );
	buttonPanel.add(   bStop  = newButton("Stop"   ,"Stop running report"  ,"STOP"   ,false) );
	if (frame!=null)
	  buttonPanel.add( bClose = newButton("Close"  ,"Close dialog"         ,"CLOSE"  ,true ) );
	buttonPanel.add(   bSave  = newButton("Save"   ,"Save report output"   ,"SAVE"   ,true) );
	buttonPanel.add(   bReload= newButton("Reload" ,"Reload report classes","RELOAD" ,true) );

	// Done
  }            
  /**
   * One of the buttons has been pressed
   */
  public void actionPerformed(ActionEvent e) {

	// Save output ?
	if (e.getActionCommand()=="SAVE") {
	  actionSave();
	  return;
	}
	
	
	// Reload  ?
	if (e.getActionCommand()=="RELOAD") {
	  tabbedPane.getModel().setSelectedIndex(0);
	  // .. do it (forced!);
	  loadReports(true);
	  // .. get them
	  Report reports[] = loader.getReports();
	  // .. update
	  listOfReports.setListData(reports);
	  // .. done
	  return;
	}

	// Stop ?
	if (e.getActionCommand()=="STOP") {
	  if (thread!=null) {
		isInterrupted=true;
		thread.interrupt();
	  }
	}
	
	// Start ?
	if (e.getActionCommand()=="START") {

	  // Calc Report
	  final Report report = (Report)listOfReports.getSelectedValue();
	  if (report!=null) {

		// .. change buttons
		setRunning(true);
		isInterrupted = false;

		// .. switch to output
		if (report.usesStandardOut())
		  tabbedPane.getModel().setSelectedIndex(1);
		taOutput.setText("");

		// .. prepare own STDOUT
		final ReportWriter writer = new ReportWriter(this);

		writer.log("<!--");
		writer.log("   Report : "+report.getClass().getName());
		writer.log("   Gedcom : "+gedcom.getName());
		writer.log("   Start  : "+new Date());
		writer.log("-->");
		writer.log("");
		writer.flush();

		// .. start Thread
		thread = new Thread() {

		  // Main Thread routine
		  public void run() {

			// Report actions are subject to interruption
			boolean rc=false;
			boolean readOnly=report.isReadOnly();
			String status="O.K.";

			// .. lock Gedcom for read and start report
			try {
				if (readOnly) {
					rc = report.start(ReportView.this,gedcom,writer);
				} else {
					if (!gedcom.startTransaction())
						status = "No Write Access";
					else {
						rc = report.start(ReportView.this,gedcom,writer);
						gedcom.endTransaction();
					}
				}
			} catch (InterruptedException ex) {
			  // Running report was stopped
			  status="Interrupted";
			} catch (Exception ex) {
			  // Running report failed
			  writer.log(ex);
			  status="Error";
			}
			isInterrupted = false;

			// .. end output
			writer.log("");
			writer.log("<!--");
			writer.log("   End    : "+new Date());
			writer.log("   Status : "+status);
			writer.log("-->");
			writer.flush();

			// .. end thread officially
			thread=null;
			
			// .. change buttons back in main thread again
			Runnable r = new Runnable() {
			  public void run() {
				setRunning(false);
			  }
			};

			SwingUtilities.invokeLater(r);

		  }

		  // End of Thread
		};
		thread.start();

		// .. done for now
	  }
	  
	  return;
	}

	// Close ?
	if (e.getActionCommand()=="CLOSE") {
	  frame.dispose();
	  return;
	}

	// Done
  }                  
  /**
   * Action SAVE
   */
  private void actionSave() {
	
	// .. choose file
	JFileChooser chooser = new JFileChooser(".");
	chooser.setDialogTitle("Save Output");
	
	if (JFileChooser.APPROVE_OPTION != chooser.showDialog(frame,"Save"))
	  return;
	File file = chooser.getSelectedFile();
	if (file==null)
	  return;
	
	// .. exits ?
	if (file.exists()) {
	  if (JOptionPane.NO_OPTION==JOptionPane.showConfirmDialog(this,"File exists. Overwrite?","Save",JOptionPane.YES_NO_OPTION))
		return;
	}

	// .. open file
	final FileWriter writer;
	try {
	  writer = new FileWriter(file);
	} catch (IOException ex) {
	  JOptionPane.showMessageDialog(this,"Error while saving to\n"+file.getAbsolutePath(),"File Error",JOptionPane.ERROR_MESSAGE);
	  return;
	}

	// .. save data
	try {

	  BufferedWriter out = new BufferedWriter(writer);
	  String data = taOutput.getText();
	  out.write(data,0,data.length());
	  out.close();

	} catch (IOException ex) {
	}

	// .. done
  }      
  public void addOutput(String line) {
	taOutput.append(line);

	if (!spOutput.getVerticalScrollBar().getValueIsAdjusting())
		taOutput.setCaretPosition(taOutput.getText().length()-1);
  }                
/**
 * Helper method that queries the user for a directory
 */
public File getDirectoryFromUser(String title, String button) {

	JFileChooser chooser = new JFileChooser();
	chooser.setFileSelectionMode(chooser.DIRECTORIES_ONLY);
	chooser.setDialogTitle(title);
	if (chooser.APPROVE_OPTION != chooser.showDialog(this,button)) {
		return null;
	}
	return chooser.getSelectedFile();
	
/*	
	Object result = JOptionPane.showInputDialog(
		this, 
		msg, 
		"Report Input",
		JOptionPane.QUESTION_MESSAGE,
		null,
		choices,
		selected);
	
	if (result==null)
	  return "";
*/	  

}
/**
 * Helper method that queries the user for a value by
 * showing a JOptionDialog.
 */
public String getValueFromUser(String msg) {
	return (String)getValueFromUser(msg,null,null);
}

/**
 * Helper method that queries the user for a value by
 * showing a JOptionDialog.
 */
public Object getValueFromUser(String msg, Object[] choices, Object selected) {
	
	Object result = JOptionPane.showInputDialog(
		this, 
		msg, 
		"Report Input",
		JOptionPane.QUESTION_MESSAGE,
		null,
		choices,
		selected);
	
	if (result==null)
	  return "";
	  
	return result;
}
  /**
   * Is this report cancelled?
   */
  public boolean isInterrupted() {
	return isInterrupted;
  }      
  /**
   * Load Reports from Disk/Net
   */
  private void loadReports(boolean force){
	
	// Reload isn't always necessary
	if ((force==false)&&(loader!=null))
	  return;

	// Create the loader
	loader = new ReportLoader(new File(System.getProperty("user.dir"),"reports"));

	// Done
  }          
  /**
   * Helper for easy button creation
   */
  private JButton newButton(String title, String tip,String action,boolean enabled) {
	JButton b = new JButton(title);
	b.setActionCommand(action);
	b.addActionListener(this);
	b.setToolTipText(tip);
	b.setEnabled(enabled);
	return b;
  }      
  /**
   * Helper that sets buttons states
   */
  private boolean setRunning(boolean on) {

	// Show it on buttons
	bStart.setEnabled(!on);
	bStop .setEnabled(on);
	if (bClose!=null)
	  bClose.setEnabled(!on);
	bReload.setEnabled(!on);

	taOutput.setCursor(Cursor.getPredefinedCursor(
	  on?Cursor.WAIT_CURSOR:Cursor.DEFAULT_CURSOR
	));
	
	// Done
	return true;
  }      
  /**
   * One of the reports in the list has been selected
   */
  public void valueChanged(ListSelectionEvent e) {
	
	// Too much selection ?
	if (e.getLastIndex()!=e.getFirstIndex()) {
	  tpInfo.setText("");
	}

	// Calc Report
	Report report = (Report)listOfReports.getSelectedValue();
	if (report==null)
	  return;

	lAuthor.setText(report.getAuthor());
	tpInfo  .setText(report.getInfo());

	// Done
	
  }        
}
