package genj.table;







import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import genj.util.GridBagHelper;import genj.gedcom.*;import genj.option.*;import genj.app.*;/**
 * Class for providing ViewInfo information to a ViewEditor
 */
public class TableViewInfo extends JPanel implements ViewInfo {

  private JComboBox       cTypes;
  private TagPathTree     pathTree;
  private TagPathList     pathList;
  private TableView       table;

  private int             eType = Gedcom.INDIVIDUALS;

  /**
   * Creates the visual parts of the editor
   */
  public TableViewInfo() {

	// Create!
	GridBagHelper helper = new GridBagHelper(this);

	// Prepare an action listener
	ActionListener alistener = new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
		if (table==null)
		  return;
		// Selection of Type?
		if (e.getSource()==cTypes) {
		  eType = cTypes.getSelectedIndex();
		  readFromTable();
		  return;
		}
		// Buttons?
		if ("UP".equals(e.getActionCommand())) {
		  pathList.up();
		}
		if ("DOWN".equals(e.getActionCommand()))
		  pathList.down();

		// Done
	  }
	};

	// Chooseable type
	cTypes = new JComboBox();
	for (int i=Gedcom.FIRST_ETYPE;i<=Gedcom.LAST_ETYPE;i++)
	  cTypes.addItem(Gedcom.getNameFor(i,true));
	cTypes.addActionListener(alistener);

	// Tree of TagPaths
	pathTree = new TagPathTree();
	TagPathTreeListener plistener = new TagPathTreeListener() {
	  public void handleSelection(TagPath path, boolean on) {
		if (!on)
		  pathList.removePath(path);
		else
		  pathList.addPath(path);
	  }
	};
	pathTree.addTagPathTreeListener(plistener);

	// List of TagPaths
	pathList = new TagPathList();

	// Up/Down of ordering
	JButton bUp   = ButtonHelper.createButton("Up"  ,null,"UP",alistener,true,false);
	JButton bDown = ButtonHelper.createButton("Down",null,"DOWN",alistener,true,false);

	// Layout
	helper.add(new JLabel("Entities")  ,0,1,1,1);
	helper.add(cTypes                  ,1,1,2,1);

	helper.add(new JLabel("Cols for")  ,0,2,1,1);
	helper.add(pathTree                ,1,2,2,1,helper.GROW_BOTH|helper.FILL_BOTH);

	helper.add(new JLabel("In order")  ,0,3,1,1);
	helper.add(bUp                     ,0,4,1,1);
	helper.add(bDown                   ,0,5,1,1);
	helper.add(pathList                ,1,3,2,4,helper.GROW_BOTH|helper.FILL_BOTH);

	// Done
  }      
  /**
   * Tells the ViewInfo to apply made changes
   */
  public void apply() {
	writeToTable();
  }      
  /**
   * Has to return the component used to edit for editing
   */
  public Component getEditor() {
	return this;
  }      
  /**
   * Reflect state of Table in our Editor
   */
  private void readFromTable() {

	// Clear reference to table for listeners
	TableView t = table;
	table = null;

	// Reflect shown type
	cTypes.setSelectedIndex(eType-Gedcom.FIRST_ETYPE);

	// Reflect columns by TagPaths
	TagPath[] selectedPaths = t.getTagPathsFor(eType);
	TagPath[] usedPaths     = TagPath.getUsedTagPaths(t.gedcom,eType);

	pathTree.setPaths(usedPaths);
	pathTree.setSelection(selectedPaths);

	pathList.setPaths(selectedPaths);

	// Show table again
	table = t;

	// Done
  }        
  /**
   * Tells the ViewInfo to reset made changes
   */
  public void reset() {
	readFromTable();
  }      
  /**
   * Initializes this ViewInfo with the given View
   */
  public void setView(Component comp) throws IllegalArgumentException {

	// Preset values
	if (!(comp instanceof TableView))
	  throw new IllegalArgumentException();

	// Calculate Table component & starting type
	table = (TableView)comp;
	eType = table.getType();

	readFromTable();

	// Done
  }        
  /**
   * Write changed state to Table
   */
  private void writeToTable() {

	// Write columns by TagPaths
	TagPath[] paths = pathList.getPaths();
	table.setTagPaths(eType,paths);

	// Done
  }      
}