package genj.timeline;



import java.awt.*;
import java.awt.event.*;

import genj.gedcom.*;/**
 * A renderer for our Timeline (lighweight) component
 *
 * POINT OF TIME            +      <br>
 * FROM TO              |-------|  <br>
 * FROM                 |----      <br>
 *      TO                   ---|  <br>
 * BET AND              [.......]  <br>
 * BEF                       ...]  <br>
 * AFT                  [...       <br>
 * ABT                      o      <br>
 * CAL                      +      <br>
 * EST                      +
 */
class TimelineContent extends Component {

  private TimelineModel model;
  private TimelineView  timeline;

  
  /**
   * Returns the logical size of the renderers view
   */
  public Dimension getPreferredSize() {
	return new Dimension(
	  model.getYears()*timeline.getPixelsPerYear(),
	  model.getLevels()*getGraphics().getFontMetrics().getHeight()*2
	);
  }      
  /**
   * Paints this content's data
   */
  public void paint(Graphics g) {

	// Fill Background
	Dimension size = getSize();
	g.setColor(Color.white);
	g.fillRect(0,0,size.width,size.height);

	// Calc some parms
	FontMetrics fm = g.getFontMetrics();
	int    minYear = model.getMinYear(),
		   maxYear = model.getMaxYear();

	// Render the links
	renderLinks(g,fm);

	// Done
  }      
  /**
   * Helper that renders the links from model
   */
  private void renderLinks(Graphics g, FontMetrics fm) {

	// Prepare painting
	int min = model.getMinYear(),
		max = model.getMaxYear();

	final TimelineGraphics tg = new TimelineGraphics(
	  g,
	  min,
	  max,
	  timeline.getPixelsPerYear(),
	  timeline.isPaintTags(),
	  timeline.isPaintDates()
	);

	// Draw Grid
	if (timeline.isPaintGrid()) {

	  g.setColor(Color.lightGray);

	  int x;
	  int h = getSize().height;
	  for (int y=min;y<=max;y++) {
		x = tg.year2pixel(y);
		g.drawLine(x,0,x,h);
	  }

	}

	// Draw Links
	g.setColor(Color.black);

	TimelineModel.Action action = new TimelineModel.Action() {
	  private float   max     = 0.0F;
	  private boolean visible = false;
	  public int doAction(Link link, boolean newLevel) {
		// Setup level of TimelineGraphics
		if (newLevel) {
		  // .. tell graphics
		  tg.incLevel();
		  // .. visible level ?
		if (!tg.isVisibleLevel())
		  return TimelineModel.Action.BREAK_LEVEL;
		  // .. prepare maximum for leftmost (first) link
		  max    = model.getMaxYear();
		  // .. initially none is visible
		  visible=false;
		}
		// Visible ?
	  if (tg.areVisibleYears(link.fromYear,max)) {
		  // .. draw visible
		  link.drawOn(tg,max);
		  // .. turn to visible mode
		  visible=true;
		  // ...
		} else {
		  // .. invisible even though we were visible already ?
		  if (visible)
			// .. end this level
			return TimelineModel.Action.BREAK_LEVEL;
		  // ...
		}
		// Maximum for next link is begin of this one
		max = link.toYear;
		// Done
		return TimelineModel.Action.CONTINUE;
	  }
	};

	model.doForAll(action);

	// Done
  }      
  /**
   * Constructor
   */
  TimelineContent(TimelineModel model, TimelineView timeline) {

	// Remember
	this.model = model;
	this.timeline = timeline;

	// Done
  }      }