package genj.timeline;







import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import genj.util.GridBagHelper;import genj.gedcom.*;import awtx.*;import genj.app.*;/**
 * Class for providing ViewInfo information to a ViewEditor
 */
public class TimelineViewInfo extends JPanel implements ViewInfo {

  private TimelineView timeline;
  private TagSelector  selectorEventTags;
  private Scala        scalaPercentPerYear;
  private JCheckBox[]  checkPaints = {
	new JCheckBox("Show Tags"),
	new JCheckBox("Show Dates"),
	new JCheckBox("Show Grid")
  };

  /**
   * Creates the visual parts of the editor
   */
  public TimelineViewInfo() {

	// Create components
	selectorEventTags = new TagSelector();
	scalaPercentPerYear = new Scala();

	// Layout
	GridBagHelper helper = new GridBagHelper(this);
	int row = 0;

	helper.add(new JLabel("Events to show"),0,row++,2,1);
	helper.addFiller(0,row,new Dimension(16,16));
	helper.add(selectorEventTags,1,row++,1,1,helper.GROW_BOTH|helper.FILL_BOTH);

	helper.add(new JLabel("Space for one year"),0,row++,2,1);
	helper.add(scalaPercentPerYear,1,row++,1,1,helper.FILL_HORIZONTAL);

	helper.add(new JLabel("Display"),0,row++,2,1);
	for (int i=0;i<checkPaints.length;i++)
	  helper.add(checkPaints[i],1,row++,1,1);

	// Done
  }      
  /**
   * Tells the ViewInfo to apply made changes
   */
  public void apply() {
	writeToTimeline();
  }      
  /**
   * Has to return the component used for editing
   */
  public Component getEditor() {
	return this;
  }      
  /**
   * Reflect state in our Editor
   */
  private void readFromTimeline() {

	// Anything to read from?
	if (timeline==null)
	  return;

	// EventTags to choose from
	String tags[] = PropertyEvent.getTags();
	selectorEventTags.setTags(tags);
	selectorEventTags.selectTags(timeline.getEventTags());

	// Space for one year
	scalaPercentPerYear.setValue(timeline.getPercentagePerYear());

	// Checks
	checkPaints[0].setSelected(timeline.isPaintTags());
	checkPaints[1].setSelected(timeline.isPaintDates());
	checkPaints[2].setSelected(timeline.isPaintGrid());

	// Done
  }        
  /**
   * Tells the ViewInfo to reset made changes
   */
  public void reset() {
	readFromTimeline();
  }      
  /**
   * Initializes this ViewInfo with the given View
   */
  public void setView(Component comp) throws IllegalArgumentException {

	// Correct type?
	if (!(comp instanceof TimelineView))
	  throw new IllegalArgumentException();

	// Calculate Timeline component & starting type
	timeline = (TimelineView)comp;
	readFromTimeline();

	// Done
  }        
  /**
   * Write changed state
   */
  private void writeToTimeline() {

	// Anything to write to?
	if (timeline==null)
	  return;

	// EventTags to choose from
	if (selectorEventTags.isChanged())
	  timeline.setEventTags(selectorEventTags.getSelectedTags());

	// Space for one year
	timeline.setPercentagePerYear(scalaPercentPerYear.getValue());

	// Checks
	timeline.setPaintTags (checkPaints[0].isSelected());
	timeline.setPaintDates(checkPaints[1].isSelected());
	timeline.setPaintGrid (checkPaints[2].isSelected());

	// Done
  }      
}