package genj.tree;


import java.awt.*;

import genj.gedcom.*;/**
 * Class that encapsulates a generic Link to an entity
 */
abstract class Link {

  private Entity entity;
  private int depthInTree;
  private int posInGen;
	
  /**
   * Constructor
   */
  Link(int setDepthInTree,int setPosInGen,Entity setEntity) {
	
	entity=setEntity;
	depthInTree=setDepthInTree;
	posInGen=setPosInGen;
	
  }      
  /**
   * Overridable action for single-click
   */
  abstract void clickIn(TreeModel model);      
  /**
   * Overridable action for double-click
   */
  abstract void dclickIn(TreeModel model);      
  /**
   * Returns the color for this link
   */
  protected Color getColor(TreeGraphics tg) {

	if (getEntity() == null)
	  return new Color(240,240,240);
	
	if (tg.isRoot(getEntity()))
	  return Color.red;
	
	return Color.black;
  }      
  /**
   * Returns the logical depth position
   */
  int getDepthInTree() {
	return depthInTree;
  }      
  /**
   * Returns the referenced entity
   */
  Entity getEntity() {
	return entity;
  }      
  /**
   * Returns the logical position in generation
   */
  int getPosInGen() {
	return posInGen;
  }      
  /**
   * Has to be overriden and returns proxies for this link
   */
  abstract Proxy[] getProxies(TreeGraphics g);      
  /**
   * Returns the pixel size in tree
   */
  abstract Dimension getSize(TreeModel model);      
  /**
   * Returns wether shadow is drawn for this linkg
   */
  boolean isShadow(TreeGraphics tg) {
	return false;
  }      
  /**
   * Move link by depth and position in generation
   */
  void moveBy(int depthChange,int posChange) {
	depthInTree+=depthChange;
	posInGen+=posChange;
  }      
  /**
   * Draws a visible link in a tree -
   * returns true if drawn in area
   */
  boolean paint(TreeGraphics g, boolean clearBG, boolean drawProps) {
	
	// Calc some parms
	Dimension s = getSize(g.getModel());
	
	int pig=getPosInGen()   ,
		dit=getDepthInTree();

	// Check Visibility
	if (!g.isVisible(dit,pig,s))
	  return false;
		  
	// Draw Shadow 
	if (isShadow(g)) {
	  g.setColor(new Color(128,128,128));
	  g.drawShadow(dit,pig,s.width,s.height);
	}	

	if (isShadow(g)||clearBG) {
	  g.setColor(Color.white);
	  g.fillRect(dit,pig,s.width,s.height);
	}
	 
	// Is this the root of the tree ?
	g.setColor(getColor(g));
	  
	// Draw Link's box
	g.drawRect(dit, pig, s.width , s.height );
	
	// Draw Properties
	if ((entity==null)||(!drawProps))
	  return true;
	  
	Proxy[] proxies = getProxies(g);
	if (proxies==null)
	  return true;
	
	g.setColor(Color.black);
	
	Property p;
	for (int i=0;i<proxies.length;i++) {
	  
	  p = entity.getProperty().getProperty(proxies[i].getPath(),true);
	  if (p!=null) {

		// Clip graphics
		Rectangle r = proxies[i].getBox();
		r.translate(-s.width/2,-s.height/2);
		g.clip(dit,pig,r);

		// Draw actual property
		try {
		  proxies[i].render(p,dit,pig,s.width,s.height,g);
		} catch (ClassCastException e)  {
		  System.out.println("Proxy "+proxies[i]+" can't render "+p);
		}

		// Restore clip
		g.restoreClip();
	  }
				
	  // Next
	}

	// Done
	return true;
  }      
  /**
   * Remembers the given entity
   */
  void setEntity(Entity setEntity) {
	entity=setEntity;
  }      
  /**
   * Sets link's position in generation
   */
  void setPosInGen(int pos) {
	posInGen=pos;
  }      
}