package genj.tree;

import java.awt.*;




import genj.gedcom.*;/**
 * Class that encapsulates a Link to a family
 */
class LinkToFam extends Link{

  private SetOfLinks children;

  /**
   * Constructor
   */
  LinkToFam(int setDepthInTree,int setPosInGen,Fam setFam) {
	super(setDepthInTree,setPosInGen,setFam);
	children = new SetOfLinks();
  }      
  /**
   * Adds a link to child to this link
   */
  void addLinkToChild(Link link) {
	children.addLink(link);
  }      
  /**
   * Action for click on PlusMinus
   */
  void clickIn(TreeModel model) {
	model.setActual( this );
  }      
  /**
   * Action for doubleclick on PlusMinus
   */
  void dclickIn(TreeModel model) {	
	model.setRoot( getEntity() );
  }      
  /**
   * Returns the family linked by this
   */
  Fam getFam() {
	return (Fam)getEntity();
  }        
  /**
   * Returns proxies for this link
   */
  Proxy[] getProxies(TreeGraphics tg) {
	return tg.famsProxies;
  }      
  /**
   * Returns the pixel size in tree
   */
  Dimension getSize(TreeModel model) {
	return model.getSize(Gedcom.FAMILIES);
  }      
  /**
   * Draws a visible family in a tree
   */
  boolean paint(TreeGraphics g, boolean clearBG, boolean drawProperties) {

	// Draw Fam's standard box
	boolean visible = super.paint(g,clearBG,drawProperties);

//    if (!drawProperties)
//      return visible;

	// Check visibility
	if (!visible) {

	  // .. standard box not visible ?
	  if (children.getSize()==0)
		// .. return when no children
		return false;
			
	  // ... calc dimension of box between family + children
	  int pig1=java.lang.Math.min( children.getLink(1).getPosInGen()                , getPosInGen() ),
		  pig2=java.lang.Math.max( children.getLink(children.getSize()).getPosInGen()  , getPosInGen() ),
		  dit1=getDepthInTree(),
		  dit2=children.getLink(1).getDepthInTree();
	    
	  // ... check if dimension lies in in rectangle
	  if (!g.isVisible(dit1,pig1,dit2,pig2))
		return false;

	  // ... visible !
	}

	// Draw line(s) to children
	g.setColor(Color.black);

	if ( (getFam().getNoOfChildren()>0) && (children.getSize()>0) ) {
	  int posInGen1 ,depthInTree1, posInGen2, depthInTree2;
	  int stub=8;

	  // ... small stub at family
	  posInGen1   = getPosInGen()   +0;
	  depthInTree1= getDepthInTree()+g.getPlusMinusOffset(Gedcom.FAMILIES);

	  posInGen2   = posInGen1;
	  depthInTree2= depthInTree1 + stub;

	  g.drawLine(depthInTree1,posInGen1,depthInTree2,posInGen2);
	  
	  // ... indication of more children
	  if (children.getSize() != getFam().getNoOfChildren()) {
		g.drawLine(depthInTree1+stub/2,posInGen1-stub,depthInTree1+stub/2,posInGen2+stub);
	  }	
	
	  // ... diagonal lines	
	  depthInTree1= depthInTree2;
	
	  for (int c=1;c<=children.getSize();c++) {
		// .. here it is
		posInGen2   = children.getLink(c).getPosInGen   ();
		depthInTree2= children.getLink(c).getDepthInTree() - g.getPlusMinusOffset(Gedcom.INDIVIDUALS) - stub;

		g.drawLine(depthInTree1,posInGen1,depthInTree2,posInGen2);
		
		// .. again small stub
		g.drawLine(depthInTree2,posInGen2,depthInTree2+stub,posInGen2);
		
		// .. next			   
	  }	
	  // Done with children
	}		

	// Done		     
	return true;

  }      
}