package genj.tree;

import java.text.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.net.URL;
import java.awt.image.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;



 




import genj.*;import genj.gedcom.*;import genj.util.*;import genj.print.*;import genj.option.*;/**
 * Class for renderering a Tree on printer
 */
public class TreeViewPrintRenderer implements PrintRenderer {

  private TreeModel       model;
  private JPanel          panelProperties;
  private float           zoom = 1.0F;
  private int             valignment = CENTER, halignment = CENTER;
  private Printer         printer;

  final static int CENTER=0, TOP=1, BOTTOM=2, LEFT=1, RIGHT=2;
  final static String valignments[] = { "Center", "Top" , "Bottom"};
  final static String halignments[] = { "Center", "Left", "Right" };


  /**
   * Constructor
   */
  public TreeViewPrintRenderer(TreeModel model) {
	this.model = (TreeModel)model.clone();
  }        
  /**
   * Returns a panel for editing this renderers properties
   */
  public JPanel getEditor() {

	// Already present ?
	if (panelProperties!=null)
	  return panelProperties;

	// Create editable fields
	final JTextField tZoom = new JTextField(5);

	final JComboBox cVertical   = new JComboBox(valignments);
	cVertical  .setSelectedIndex(valignment);
	final JComboBox cHorizontal = new JComboBox(halignments);
	cHorizontal.setSelectedIndex(halignment);

	readFromUser(tZoom,cVertical,cHorizontal);

	// Prepare ActionListener
	ActionListener alistener = new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
		readFromUser(tZoom,cVertical,cHorizontal);
	  }
	};
	tZoom      .addActionListener(alistener);
	cVertical  .addActionListener(alistener);
	cHorizontal.addActionListener(alistener);

	// Create panel
	panelProperties = new JPanel();
	panelProperties.add(new JLabel("Zoom"));
	panelProperties.add(tZoom);
	panelProperties.add(new JLabel("Vertical"));
	panelProperties.add(cVertical);
	panelProperties.add(new JLabel("Horizontal"));
	panelProperties.add(cHorizontal);

	// Done
	return panelProperties;
  }        
  /**
   * Returns size of rendering object in pixels
   */
  public Dimension getSize() {
	return new Dimension(
	  (int)(model.getSize().width *zoom),
	  (int)(model.getSize().height*zoom)
	);
  }        
  /**
   * Helper that reads from user input components
   */
  private void readFromUser(JTextField tZoom, JComboBox cVertical, JComboBox cHorizontal) {

	// Parse Zoom
	NumberFormat format = NumberFormat.getPercentInstance();
	Number newZoom;
	try {
	  newZoom = format.parse(tZoom.getText());
	  zoom = Math.max(0.05F, newZoom.floatValue());
	} catch (ParseException e) {
	}
	tZoom.setText(format.format(zoom));

	// Parse Vertical alignment
	valignment = cVertical.getSelectedIndex();

	// Parse Horizontal alignment
	halignment = cHorizontal.getSelectedIndex();

	// Tell to printer
	if (printer!=null)
	  printer.rendererChanged();

	// Done
  }        
/**
 * Renders print data for page
 * @param g Graphics to render on
 * @param dimPages Dimension of all pages printed
 */
public void renderPage(Graphics g, Dimension dimPages) {

	// Calculate offset for alignment
	int xoffset, yoffset;

	switch (halignment) {
	  case TOP   :
		xoffset = 0;
		break;
	  default:
	  case CENTER:
		xoffset = (int)( dimPages.width /2 - getSize().width/2);
		break;
	  case BOTTOM:
		xoffset = (int)( dimPages.width    - getSize().width  );
		break;
	}
	switch (valignment) {
	  case LEFT  :
		yoffset = 0;
		break;
	  default:
	  case CENTER:
		yoffset = (int)( dimPages.height/2 - getSize().height/2);
		break;
	  case RIGHT :
		yoffset = (int)( dimPages.height   - getSize().height  );
		break;
	}

	// Create Graphics for model drawing
	g.translate(xoffset,yoffset);

	final TreeGraphics tg = new TreeGraphics(
	  g,
	  model,
	  zoom,
	  false,
	  false,
	  false,
	  true
	);

	Enumeration links = model.getLinks();
	while (links.hasMoreElements())
		((Link)links.nextElement()).paint(tg,false,true);

	// Done
}
  /**
   * Rendering of preview
   */
  public void renderPreview(Graphics g, Dimension dimPreview, float zoomPreview) {

	// Calculate offset for alignment
	int xoffset, yoffset;

	switch (halignment) {
	  case TOP   :
		xoffset = 0;
		break;
	  default:
	  case CENTER:
		xoffset = (int)( dimPreview.width /2 - (getSize().width *zoomPreview)/2);
		break;
	  case BOTTOM:
		xoffset = (int)( dimPreview.width    - (getSize().width *zoomPreview)  );
		break;
	}
	switch (valignment) {
	  case LEFT  :
		yoffset = 0;
		break;
	  default:
	  case CENTER:
		yoffset = (int)( dimPreview.height/2 - (getSize().height*zoomPreview)/2);
		break;
	  case RIGHT :
		yoffset = (int)( dimPreview.height   - (getSize().height*zoomPreview)  );
		break;
	}

	// Create Graphics for model drawing
	g.translate(xoffset,yoffset);

	final TreeGraphics tg = new TreeGraphics(
	  g,
	  model,
	  zoom * zoomPreview,
	  false,
	  false,
	  false,
	  false
	);

	Enumeration links = model.getLinks();
	while (links.hasMoreElements())
		((Link)links.nextElement()).paint(tg,false,false);

	// Done

  }        
  /**
   * Sets the printer which is using this renderer
   */
  public void setPrinter(Printer printer) {
	this.printer = printer;
  }        
}