package genj.util;

import java.awt.*;
import java.net.*;
import java.io.*;
import java.util.*;
import javax.swing.ImageIcon;

/**
 * Class that represents an improved image
 */
public class ImgIcon {

  private Object           swingicon=null;
  private Image            image=null;
  private Dimension        size;
  private static Hashtable zoomed = new Hashtable();

  private final static Component component = new TextField();
  private final static MediaTracker tracker = new MediaTracker(component);
  private final static int CLUSTER = 1024*4;

  /**
   * Constructor #4
   */
  public ImgIcon(Image image) {

	// Remember image
	this.image = image;

	// Calculate dimension
	size = calcDimension(image);

	// Done
  }      
  /**
   * Constructor #2
   */
  public ImgIcon(InputStream in) throws IOException {

	// Load it
	createImageFrom(in);

	// Done
  }      
  /**
   * Constructor #3
   */
  public ImgIcon(Object from, String name) {

	// Get InputStream
	InputStream in = from.getClass().getResourceAsStream(name);
	// Load it
	createImageFrom(in);

	// Done
  }      
  /**
   * Constructor #1
   */
  public ImgIcon(URL url) throws IOException {

	// Load it
	createImageFrom(url.openStream());

	// Done
  }      
  /**
   * Calculates dimension of image (eventually waiting)
   */
  private static Dimension calcDimension(Image image) {

	// Wait till ready
	synchronized (tracker) {
	  tracker.addImage(image,0);

	  try {
		tracker.waitForID(0,5000);
	  } catch (InterruptedException ex) {
		System.out.println("Interrupted while loading image");
		return new Dimension(0,0);
	  } finally {
		tracker.removeImage(image,0);
	  }
	}

	// Calc width/height for result
	// 03.02.2000 2nd parameter should be getHeight and not getWidth :)
	return new Dimension(image.getWidth(null),image.getHeight(null));
	
  }      
  /**
   * Helper which actually loads the image
   */
  private void createImageFrom(InputStream in) {

	// Read from stream
	byte imgData[] = new byte[CLUSTER];
	int len=0,total=0;

	while (true) {

	  // Read !
	  try {
		len = in.read(imgData,total,imgData.length-total);
	  } catch (IOException ex) {
		return;
	  }

	  // End of stream ?
	  if (len<0)
		break;

	  // Increment amount read !
	  total+=len;

	  // Did it fit and end ?
	  if (total<imgData.length) {
		// NM 19990109 : Changed break to continue
		continue;
	  }

	  // More than fit !
	  byte tmp[] = new byte[imgData.length*2];
	  System.arraycopy(imgData,0,tmp,0,imgData.length);
	  imgData = tmp;

	  // Read on !
	}

	// Create image
	image = Toolkit.getDefaultToolkit().createImage(imgData,0,total);

	// Calculate dimension
	size = calcDimension(image);

	// Done
	imgData=null;
  }      
  /**
   * Returns image's height
   */
  public int getIconHeight() {
	return size.height;
  }      
  /**
   * Returns image's width
   */
  public int getIconWidth() {
	return size.width;
  }      
  /**
   * Returns this image
   */
  public Image getImage() {
	return image;
  }      
  /**
   * Returns a (cached) Swing ImageIcon representation of this image
   */
  public ImageIcon getImageIcon() {

	// Already present ?
	if (swingicon!=null)
	  return (ImageIcon)swingicon;

	// Make it
	swingicon = new ImageIcon(image);

	// Done
	return (ImageIcon)swingicon;
  }      
  /**
   * Paints the image
   */
  public void paintIcon(Component c,Graphics g, int x, int y) {
	g.drawImage(image,x,y,c);
  }      
/**
 * Paints the image
 */
public void paintIcon(Component c,Graphics g, int x, int y, double zoom) {

	// No real zoom ?
	if (zoom==1.0) {
	  paintIcon(c,g,x,y);
	  return;
	}

	// Calc w & h
	int w = (int)(size.width*zoom),
		h = (int)(size.height*zoom);

	// Known Image ?
	String key = image+"/"+w+"x"+h;
	Object o = zoomed.get(key);
	
	Image i;
	if (o!=null) {
	  i = (Image)o;
	} else {
	
	  // Create Image
	  i = image.getScaledInstance( w, h, Image.SCALE_DEFAULT);
	  calcDimension(i);
	  
	  // Remember
	  zoomed.put(key,i);
	}

	// Paint
	g.drawImage(i,x,y,null);

	// Done
  }      
  /**
   * Paints the image
   */
  public void paintIcon(Graphics g, int x, int y) {
	g.drawImage(image,x,y,null);
  }      
  /**
   * Paints the image
   */
  public void paintIcon(Graphics g, int x, int y, double zoom) {
	paintIcon(null,g,x,y,zoom);
  }      
}