package genj.util;

import java.util.*;
import java.text.MessageFormat;

/**
 * Class which provides localized text-resources for a package
 */
public class Resources {

  private ResourceBundle rb;
  private String pkg;
  private MessageFormat format;
  
  /**
   * Constructor
   */
  public Resources(Object packageMember) {

	// Prepare information
	String me = packageMember.getClass().getName();
	String pkg = me.substring(0, me.lastIndexOf('.'));

	init(pkg);

  }      
  /**
   * Constructor
   */
  public Resources(String pkg) {
	init(pkg);
  }      
  /**
   * Returns a localized string
   * @param key identifies string to return
   */
  public String getString(String key) {
	
	try {
	  
	  return rb.getString(key);
	  
	} catch (RuntimeException e) {
	  
	  System.out.println("Resource "+key+" for pkg "+pkg+" is missing");

	  return "";
	  
	}
  }      
  /**
   * Returns a localized string
   * @param key identifies string to return
   * @param values array of values to replace placeholders in value
   */
  public String getString(String key, Object substitute) {
	
	try {

	  // Get Value
	  String value = rb.getString(key);

	  // Replace with substitutes
	  format.applyPattern(value);
	  Object[] subs = { substitute };
	  return format.format(subs);
	  
	} catch (RuntimeException e) {
	  
	  System.out.println("Resource "+key+" for pkg "+pkg+" is missing");

	  return "";
	}
  }      
  /**
   * Init
   */
  private void init(String pkg) {

	this.format=new MessageFormat("");
	this.pkg=pkg;
	
	try {
	  
	  rb = ResourceBundle.getBundle(pkg+".resources",Locale.getDefault());
	  
	} catch (RuntimeException e) {
	  
	  System.out.println("Error reading resources from "+pkg+": "+e.getMessage());
	  throw e;
	  
	}

	// Done
  }        
}